SystemOrganization addCategory: #'ToolBuilder-Morphic'!

ToolBuilder subclass: #MorphicToolBuilder
	instanceVariableNames: 'widgets panes parentMenu'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!MorphicToolBuilder commentStamp: 'ar 2/11/2005 15:02' prior: 0!
The Morphic tool builder.!

!MorphicToolBuilder class methodsFor: 'services' stamp: 'mt 5/4/2016 13:21'!
getBoundsForWindow: morph

	^ morph bounds! !

!MorphicToolBuilder class methodsFor: 'accessing' stamp: 'ar 2/11/2005 15:24'!
isActiveBuilder
	"Answer whether I am the currently active builder"
	^Smalltalk isMorphic! !

!MorphicToolBuilder class methodsFor: 'services' stamp: 'mt 5/4/2016 13:21'!
setBoundsForWindow: morph to: rectangle

	^ morph bounds: rectangle! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'mt 4/28/2015 15:01'!
add: aMorph to: aParent
	
	aMorph layoutFrame
		ifNotNil: [aParent addMorph: aMorph fullFrame: aMorph layoutFrame]
		ifNil: [aParent addMorphBack: aMorph].! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'cmm 3/2/2010 17:19'!
alternateMultiSelectListClass
	^ AlternatePluggableListMorphOfMany ! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'stephaneducasse 2/3/2006 22:35'!
asFrame: aRectangle
	| frame |
	aRectangle ifNil:[^nil].
	frame := LayoutFrame new.
	frame 
		leftFraction: aRectangle left; 
		rightFraction: aRectangle right; 
		topFraction: aRectangle top; 
		bottomFraction: aRectangle bottom.
	^frame! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'mt 3/30/2015 09:12'!
buildHelpFor: widget spec: aSpec
	aSpec help ifNotNil: [:stringOrSymbol |
		stringOrSymbol isSymbol
			ifTrue: [widget balloonTextSelector: stringOrSymbol]
			ifFalse: [widget balloonText: stringOrSymbol]].! !

!MorphicToolBuilder methodsFor: 'widgets optional' stamp: 'mt 4/5/2016 10:28'!
buildPluggableAlternateMultiSelectionList: aSpec
	| listMorph listClass |
	aSpec getSelected ifNotNil: [ ^ self error: 'There is no PluggableAlternateListMorphOfManyByItem' ].
	listClass := self alternateMultiSelectListClass.
	listMorph := listClass 
		on: aSpec model
		list: aSpec list
		primarySelection: aSpec getIndex
		changePrimarySelection: aSpec setIndex
		listSelection: aSpec getSelectionList
		changeListSelection: aSpec setSelectionList
		menu: aSpec menu.
	listMorph
		setProperty: #highlightSelector toValue: #highlightMessageList:with: ;
		setProperty: #itemConversionMethod toValue: #asStringOrText ;
		setProperty: #balloonTextSelectorForSubMorphs toValue: #balloonTextForClassAndMethodString ;
		enableDragNDrop: SystemBrowser browseWithDragNDrop ;
		menuTitleSelector: #messageListSelectorTitle.
	self 
		register: listMorph
		id: aSpec name.
	listMorph
		keystrokeActionSelector: aSpec keyPress ;
		getListElementSelector: aSpec listItem ;
		getListSizeSelector: aSpec listSize;
		getIconSelector: aSpec icon;
		getHelpSelector: aSpec helpItem.
		
	self buildHelpFor: listMorph spec: aSpec.
		
	self 
		setFrame: aSpec frame 
		in: listMorph.
	self setLayoutHintsFor: listMorph spec: aSpec.
	parent ifNotNil: [ self add: listMorph to: parent ].
	panes ifNotNil: [ aSpec list ifNotNil:[panes add: aSpec list ] ].
	^ listMorph! !

!MorphicToolBuilder methodsFor: 'widgets required' stamp: 'nice 6/10/2017 17:33'!
buildPluggableButton: aSpec
	| widget label state action enabled |
	label := aSpec label.
	state := aSpec state.
	action := aSpec action.
	widget := self buttonClass on: aSpec model
				getState: (state isSymbol ifTrue:[state])
				action: nil
				label: (label isSymbol ifTrue:[label]).
	widget style: aSpec style.
	aSpec changeLabelWhen
		ifNotNil: [ :event | widget whenChanged: event update: aSpec label].
	self register: widget id: aSpec name.
	enabled := aSpec enabled.
	enabled isSymbol
		ifTrue:[widget getEnabledSelector: enabled]
		ifFalse:[widget enabled:enabled].
	widget action: action.
	aSpec color isColor
		ifTrue: [widget offColor: aSpec color]
		ifFalse: [widget getColorSelector: aSpec color].
	self buildHelpFor: widget spec: aSpec. 
	(label isSymbol or:[label == nil]) ifFalse:[widget label: label].
	self setFrame: aSpec frame in: widget.
	self setLayoutHintsFor: widget spec: aSpec.
	parent ifNotNil:[self add: widget to: parent].
	^widget! !

!MorphicToolBuilder methodsFor: 'widgets optional' stamp: 'mt 4/24/2015 10:29'!
buildPluggableCheckBox: spec

	| widget label state action |
	label := spec label.
	state := spec state.
	action := spec action.
	widget := self checkBoxClass on: spec model
				getState: (state isSymbol ifTrue:[state])
				action: (action isSymbol ifTrue:[action])
				label: (label isSymbol ifTrue:[label]).
	self register: widget id: spec name.

	widget installButton.
"	widget getColorSelector: spec color.
	widget offColor: Color white..
	self buildHelpFor: widget spec: spec. 
	(label isSymbol or:[label == nil]) ifFalse:[widget label: label].
"	self setFrame: spec frame in: widget.
	self setLayoutHintsFor: widget spec: spec.
	parent ifNotNil:[self add: widget to: parent].
	^widget! !

!MorphicToolBuilder methodsFor: 'widgets optional' stamp: 'mt 4/28/2015 12:12'!
buildPluggableCodePane: aSpec

	| widget |
	widget := super buildPluggableCodePane: aSpec.
	
	"Override code-specific default properties."
	widget wantsWrapBorder: PluggableTextMorph visualWrapBorder.
	widget wrapFlag: (aSpec softLineWrap ifNil: [PluggableTextMorph softLineWrap]).
	^ widget! !

!MorphicToolBuilder methodsFor: 'widgets optional' stamp: 'mt 8/13/2016 09:51'!
buildPluggableDialog: aSpec

	| widget |

	widget := self dialogClass new.
	self register: widget id: aSpec name.
	
	widget model: aSpec model.

	"Set child dependent layout properties. The pane morph holds the special contents."
	widget paneMorph wantsPaneSplitters: (aSpec wantsResizeHandles ifNil: [true]).
	self setLayoutHintsFor: widget paneMorph spec: aSpec.
	widget paneMorph layoutInset: (aSpec padding ifNil: [ProportionalSplitterMorph gripThickness]).
	widget paneMorph cellInset: (aSpec spacing ifNil: [ProportionalSplitterMorph gripThickness]).
	widget paneMorph wantsPaneSplitters ifTrue: [
		widget paneMorph addCornerGrips"addEdgeGrips".
		widget paneMorph grips do: [:ea | ea drawCornerResizeHandles: true]].

	"Now create the children."
	panes := OrderedCollection new.
	aSpec children isSymbol
		ifTrue: [
			widget getChildrenSelector: aSpec children.
			widget update: aSpec children]
		ifFalse: [
			self buildAll: aSpec children in: widget paneMorph].

	"Now create the buttons."
	aSpec buttons isSymbol
		ifTrue: [
			widget getButtonsSelector: aSpec buttons.
			widget update: aSpec buttons]
		ifFalse: [
			self buildAll: aSpec buttons in: widget buttonRowMorph.
			widget updateButtonProperties].

	aSpec title ifNotNil: [:label |
		label isSymbol 
			ifTrue:[widget getTitleSelector: label; update: label]
			ifFalse:[widget title: label]].
	aSpec message ifNotNil: [:label |
		label isSymbol 
			ifTrue:[widget getMessageSelector: label; update: label]
			ifFalse:[widget message: label]].
	
	"Interaction behavior."
	aSpec autoCancel ifNotNil: [:b | widget autoCancel: b].
	aSpec exclusive ifNotNil: [:b | widget exclusive: b].
		
	widget closeDialogSelector: aSpec closeAction.
	self buildHelpFor: widget spec: aSpec. 

	"Everything is shrink-wrapped around the pane morph."
	widget paneMorph extent: (aSpec extent ifNil:[widget initialExtent]).

	^ widget! !

!MorphicToolBuilder methodsFor: 'widgets optional' stamp: 'mt 4/24/2015 10:29'!
buildPluggableDropDownList: spec

	| widget model listSelector selectionSelector selectionSetter |
	model := spec model.
	listSelector := spec listSelector.
	selectionSelector := spec selectionSelector.
	selectionSetter := spec selectionSetter.
	widget := self dropDownListClass new
		model: model;
		listSelector: listSelector;
		selectionSelector: selectionSelector;
		selectionSetter: selectionSetter;
		yourself.
	self register: widget id: spec name.

	widget installDropDownList.
	self setFrame: spec frame in: widget.
	self setLayoutHintsFor: widget spec: spec.
	parent ifNotNil:[self add: widget to: parent].
	^widget! !

!MorphicToolBuilder methodsFor: 'widgets optional' stamp: 'mt 4/27/2015 15:46'!
buildPluggableInputField: aSpec
	| widget |
	widget := self buildPluggableText: aSpec.
	widget minimumHeight: 0.
	widget acceptOnCR: true.
	widget hideScrollBarsIndefinitely.
	^widget! !

!MorphicToolBuilder methodsFor: 'widgets required' stamp: 'topa 1/17/2017 17:54'!
buildPluggableList: aSpec
	| widget listClass getIndex setIndex |
	aSpec getSelected ifNil:[
		listClass := self listClass.
		getIndex := aSpec getIndex.
		setIndex := aSpec setIndex.
	] ifNotNil:[
		listClass := self listByItemClass.
		getIndex := aSpec getSelected.
		setIndex := aSpec setSelected.
	].
	widget := listClass on: aSpec model
				list: aSpec list
				selected: getIndex
				changeSelected: setIndex
				menu: aSpec menu
				keystroke: aSpec keyPress.
	self register: widget id: aSpec name.
	
	widget
		dragItemSelector: aSpec dragItem;
		dropItemSelector: aSpec dropItem;
		wantsDropSelector: aSpec dropAccept;
		dragStartedSelector: aSpec dragStarted.

	self setListPropertiesFor: widget spec: aSpec.
	^widget! !

!MorphicToolBuilder methodsFor: 'widgets required' stamp: 'Igor.Stasenko 1/17/2010 04:20'!
buildPluggableMenu: menuSpec 
	| prior menu |
	prior := parentMenu.
	parentMenu := menu := self menuClass new.
	menuSpec label ifNotNil:[parentMenu addTitle: menuSpec label].
	menuSpec items do:[:each| each buildWith: self].
	parentMenu := prior.
	^menu! !

!MorphicToolBuilder methodsFor: 'widgets required' stamp: 'fbs 6/18/2013 17:31'!
buildPluggableMenuItem: itemSpec
	| item action label menu |
	item := self menuItemClass new.
	label := (itemSpec isCheckable
		ifTrue: [	itemSpec checked ifTrue: ['<on>'] ifFalse: ['<off>']]
		ifFalse: ['']), itemSpec label.
	item contents: label.
	item isEnabled: itemSpec enabled.
	(action := itemSpec action) ifNotNil:[
		item 
			target: action receiver;
			selector: action selector;
			arguments: action arguments.
	].
	(menu := itemSpec subMenu) ifNotNil:[
		item subMenu: (menu buildWith: self).
	].
	parentMenu ifNotNil:[parentMenu addMorphBack: item].
	itemSpec separator ifTrue:[parentMenu addLine].
	^item! !

!MorphicToolBuilder methodsFor: 'widgets optional' stamp: 'topa 1/17/2017 17:51'!
buildPluggableMultiSelectionList: aSpec
	| widget listClass |
	aSpec getSelected ifNotNil:[^self error:'There is no PluggableListMorphOfManyByItem'].
	listClass := self multiSelectListClass.
	widget := listClass on: aSpec model
		list: aSpec list
		primarySelection: aSpec getIndex
		changePrimarySelection: aSpec setIndex
		listSelection: aSpec getSelectionList
		changeListSelection: aSpec setSelectionList
		menu: aSpec menu.
	widget keystrokeActionSelector: aSpec keyPress.
	self register: widget id: aSpec name.
	self setListPropertiesFor: widget spec: aSpec.
	^widget! !

!MorphicToolBuilder methodsFor: 'widgets required' stamp: 'mt 4/30/2015 14:22'!
buildPluggablePanel: aSpec

	| widget |
	widget := self panelClass new.
	self register: widget id: aSpec name.

	widget model: aSpec model.

	"Set child dependent layout properties."
	widget wantsPaneSplitters: (aSpec wantsResizeHandles ifNil: [false]).
	self setLayoutHintsFor: widget spec: aSpec.
	widget layoutInset: (aSpec padding ifNil: [0]).
	widget cellInset: (aSpec spacing ifNil: [
		widget wantsPaneSplitters
			ifTrue: [ProportionalSplitterMorph gripThickness]
			ifFalse: [2]]).

	"Now create the children."
	aSpec children isSymbol
		ifTrue: [
			widget getChildrenSelector: aSpec children.
			widget update: aSpec children]
		ifFalse: [
			self buildAll: aSpec children in: widget].

	self setFrame: aSpec frame in: widget.
	self setLayout: aSpec layout in: widget.
	
	parent ifNotNil:[self add: widget to: parent].

	widget borderWidth: 0.
	self buildHelpFor: widget spec: aSpec. 
	widget color: Color transparent.
	
	^ widget! !

!MorphicToolBuilder methodsFor: 'widgets optional' stamp: 'mt 7/26/2016 11:24'!
buildPluggableScrollPane: spec

	| widget panel |
	widget := self scrollPaneClass new.
	widget model: spec model.
	self register: widget id: spec name.
	
	spec children
		ifNotNil: [:obj |
			"Create panel morph to scroll it."
			panel := self pluggablePanelSpec new
				model: spec model;
				children: obj;
				layout: spec layout;
				horizontalResizing: (spec layout == #proportional ifTrue: [#rigid] ifFalse: [#shrinkWrap]);
				verticalResizing: (spec layout == #proportional ifTrue: [#rigid] ifFalse: [#shrinkWrap]);
				buildWith: self.
			widget morph: panel.
			]
		ifNil: [spec morph
			ifNotNil: [:m | widget morph: m]
			ifNil: [widget morphClass: spec morphClass]].
	
	self setFrame: spec frame in: widget.
	self setLayoutHintsFor: widget spec: spec.
	
	parent ifNotNil: [self add: widget to: parent].
	
	spec borderWidth ifNotNil: [:w | widget borderWidth: w].
	
	"Override default scroll bar policies if needed. Widget will use preference values otherwise."
	spec hScrollBarPolicy ifNotNil: [:policy |
		policy caseOf: {
			[#always] -> [widget alwaysShowHScrollBar].
			[#never] -> [widget hideHScrollBarIndefinitely].
			[#whenNeeded] -> [widget showHScrollBarOnlyWhenNeeded]}].
	spec vScrollBarPolicy ifNotNil: [:policy | 
		policy caseOf: {
			[#always] -> [widget alwaysShowVScrollBar].
			[#never] -> [widget hideVScrollBarIndefinitely].
			[#whenNeeded] -> [widget showVScrollBarOnlyWhenNeeded]}].
	
	^ widget! !

!MorphicToolBuilder methodsFor: 'widgets required' stamp: 'topa 7/8/2015 17:34'!
buildPluggableSpacer: spec

	| widget |
	widget := self spacerClass new.
	self register: widget id: spec name.
	
	widget borderWidth: 0.
	widget color: (spec color ifNil: [Color transparent]).
	widget extent: spec extent.

	self buildHelpFor: widget spec: spec. 
	
	self setFrame: spec frame in: widget.
	self setLayoutHintsFor: widget spec: spec.
		
	parent ifNotNil:[self add: widget to: parent].
	^widget! !

!MorphicToolBuilder methodsFor: 'widgets required' stamp: 'mt 8/16/2016 13:18'!
buildPluggableText: aSpec
	| widget |
	widget := self textPaneClass new.
	aSpec stylerClass ifNotNil: [:c | widget styler: (c new view: widget)].	
	widget
		on: aSpec model
		text: aSpec getText 
		accept: aSpec setText
		readSelection: aSpec selection 
		menu: aSpec menu.
	aSpec font ifNotNil: [:f | widget font: f].
	
	widget readOnly: aSpec readOnly.
	widget editTextSelector: aSpec editText.
	widget wantsFrameAdornments: aSpec indicateUnacceptedChanges.

	widget askBeforeDiscardingEdits: aSpec askBeforeDiscardingEdits.
	widget setProperty: #alwaysAccept toValue: aSpec askBeforeDiscardingEdits not.

	aSpec softLineWrap ifNotNil: [:b | widget wrapFlag: b].

	widget isAutoFit
		ifTrue: [widget hideHScrollBarIndefinitely]
		ifFalse: [widget showHScrollBarOnlyWhenNeeded].
	
	self register: widget id: aSpec name.
	widget getColorSelector: aSpec color.
	widget getTextColorSelector: aSpec textColor.
	self buildHelpFor: widget spec: aSpec.
	 
	self setFrame: aSpec frame in: widget.
	self setLayoutHintsFor: widget spec: aSpec.
	
	parent ifNotNil:[self add: widget to: parent].
	^widget! !

!MorphicToolBuilder methodsFor: 'widgets required' stamp: 'mt 11/4/2015 14:22'!
buildPluggableTree: aSpec
	| widget |
	widget := self treeClass new.
	self register: widget id: aSpec name.
	widget model: aSpec model.
	widget getSelectedPathSelector: aSpec getSelectedPath.
	widget setSelectedSelector: aSpec setSelected.
	widget getSelectedSelector: aSpec getSelected.
	widget setSelectedParentSelector: aSpec setSelectedParent.
	widget getChildrenSelector: aSpec getChildren.
	widget hasChildrenSelector: aSpec hasChildren.
	widget getLabelSelector: aSpec label.
	widget getIconSelector: aSpec icon.
	widget getHelpSelector: aSpec help.
	widget getMenuSelector: aSpec menu.
	widget keystrokeActionSelector: aSpec keyPress.
	widget nodeClass: aSpec nodeClass.
	widget getRootsSelector: aSpec roots.
	widget autoDeselect: aSpec autoDeselect.
	
	widget dropItemSelector: aSpec dropItem.
	widget wantsDropSelector: aSpec dropAccept.
	widget dragItemSelector: aSpec dragItem.
	widget dragStartedSelector: aSpec dragStarted.
	
	widget columns: aSpec columns.
	
	"Override default scroll bar policies if needed. Widget will use preference values otherwise."
	aSpec hScrollBarPolicy ifNotNil: [:policy |
		policy caseOf: {
			[#always] -> [widget alwaysShowHScrollBar].
			[#never] -> [widget hideHScrollBarIndefinitely].
			[#whenNeeded] -> [widget showHScrollBarOnlyWhenNeeded]. } ].
	aSpec vScrollBarPolicy ifNotNil: [:policy |
		policy caseOf: {
			[#always] -> [widget alwaysShowVScrollBar].
			[#never] -> [widget hideVScrollBarIndefinitely].
			[#whenNeeded] -> [widget showVScrollBarOnlyWhenNeeded]. } ].	
	
	self setFrame: aSpec frame in: widget.
	self setLayoutHintsFor: widget spec: aSpec.
	
	parent ifNotNil:[self add: widget to: parent].
"	panes ifNotNil:[
		aSpec roots ifNotNil:[panes add: aSpec roots].
	].	"
	^widget! !

!MorphicToolBuilder methodsFor: 'widgets required' stamp: 'mt 4/30/2015 14:22'!
buildPluggableWindow: aSpec
	| widget |

	aSpec layout == #proportional ifFalse:[
		"This needs to be implemented - probably by adding a single pane and then the rest"
		^self error: 'Not implemented'.
	].

	widget := (self windowClassFor: aSpec) new.
	self register: widget id: aSpec name.
	
	widget model: aSpec model.

	"Set child dependent layout properties."
	widget wantsPaneSplitters: (aSpec wantsResizeHandles ifNil: [true]).
	self setLayoutHintsFor: widget spec: aSpec.
	widget layoutInset: (aSpec padding ifNil: [ProportionalSplitterMorph gripThickness]).
	widget cellInset: (aSpec spacing ifNil: [ProportionalSplitterMorph gripThickness]).
	
	"Now create the children."
	panes := OrderedCollection new.
	aSpec children isSymbol
		ifTrue: [
			widget getChildrenSelector: aSpec children.
			widget update: aSpec children]
		ifFalse: [
			self buildAll: aSpec children in: widget].
	widget setUpdatablePanesFrom: panes.
	
	aSpec label ifNotNil: [:label|
		label isSymbol 
			ifTrue:[widget getLabelSelector: label]
			ifFalse:[widget setLabel: label]].

	aSpec multiWindowStyle notNil ifTrue:
		[widget savedMultiWindowState: (SavedMultiWindowState on: aSpec model)].

	widget closeWindowSelector: aSpec closeAction.
	self buildHelpFor: widget spec: aSpec. 

	widget bounds: (RealEstateAgent 
		initialFrameFor: widget 
		initialExtent: (aSpec extent ifNil:[widget initialExtent])
		world: self currentWorld).

	^ widget! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'Igor.Stasenko 1/17/2010 04:01'!
buttonClass
	^ PluggableButtonMorphPlus! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'rww 9/11/2010 03:39'!
checkBoxClass
	^ PluggableCheckBoxMorph! !

!MorphicToolBuilder methodsFor: 'opening' stamp: 'ar 6/5/2005 12:40'!
close: aWidget
	"Close a previously opened widget"
	aWidget delete! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'Igor.Stasenko 1/17/2010 04:03'!
codePaneClass
	^ PluggableTextMorphPlus! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'mt 7/13/2016 16:40'!
dialogClass
	^ PluggableDialogWindow! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'rww 9/11/2010 14:05'!
dropDownListClass
	^ PluggableDropDownListMorph! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'Igor.Stasenko 1/17/2010 04:11'!
listByItemClass
	^ PluggableListMorphByItemPlus! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'Igor.Stasenko 1/17/2010 04:06'!
listClass
	^ PluggableListMorphPlus! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'Igor.Stasenko 1/17/2010 04:20'!
menuClass
	^ MenuMorph! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'Igor.Stasenko 1/17/2010 04:20'!
menuItemClass
	^ MenuItemMorph! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'Igor.Stasenko 1/17/2010 04:14'!
multiSelectListClass
	^ PluggableListMorphOfMany! !

!MorphicToolBuilder methodsFor: 'opening' stamp: 'mt 10/5/2017 08:46'!
open: anObject
	"Build and open the object. Answer the widget opened."
	
	^ (self build: anObject) openAsTool! !

!MorphicToolBuilder methodsFor: 'opening' stamp: 'mt 5/11/2016 13:51'!
open: anObject label: aString
	"Build an open the object, labeling it appropriately if it is a window. Answer the widget opened."
	
	| morph |
	morph := self open: anObject.
	morph isSystemWindow
		ifTrue: [morph setLabel: aString]
		ifFalse: [morph name: aString].
	^ morph! !

!MorphicToolBuilder methodsFor: 'opening' stamp: 'mt 6/2/2016 18:28'!
openDebugger: aSpec

	| morph |
	morph := self build: aSpec.
	morph openInWorld: Project current world extent: morph extent.
	^ morph! !

!MorphicToolBuilder methodsFor: 'opening' stamp: 'mt 6/2/2016 18:25'!
openDebugger: aSpec label: aString

	| morph |
	morph := self openDebugger: aSpec.
	morph isSystemWindow
		ifTrue: [morph setLabel: aString]
		ifFalse: [morph name: aString].
	^ morph! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'Igor.Stasenko 1/17/2010 04:14'!
panelClass
	^ PluggablePanelMorph! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'ar 9/6/2009 16:14'!
register: widget id: id
	id ifNil:[^self].
	widgets ifNil:[widgets := Dictionary new].
	widgets at: id put: widget.
	widget setNameTo: id.! !

!MorphicToolBuilder methodsFor: 'opening' stamp: 'ar 6/5/2005 12:41'!
runModal: aWidget
	"Run the (previously opened) widget modally, e.g., 
	do not return control to the sender before the user has responded."
	[aWidget world notNil] whileTrue: [
		aWidget outermostWorldMorph doOneCycle.
	].
! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'mt 6/15/2014 07:24:33.104'!
scrollPaneClass
	^ PluggableScrollPane! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'mt 4/14/2015 16:44'!
setFrame: aRectangle in: widget
	| frame |
	aRectangle ifNil:[^nil].
	frame := aRectangle isRectangle
		ifTrue: [self asFrame: aRectangle]
		ifFalse: [aRectangle]. "assume LayoutFrame"
	widget layoutFrame: frame.
	widget hResizing: #spaceFill; vResizing: #spaceFill.! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'mt 5/3/2015 10:20'!
setLayout: layout in: widget
	layout == #proportional ifTrue:[
		widget layoutPolicy: ProportionalLayout new.
		^self].
	layout == #horizontal ifTrue:[
		widget layoutPolicy: TableLayout new.
		widget listDirection: #leftToRight.
		"widget cellInset: 1@1.
		widget layoutInset: 1@1."
		widget color: Color transparent.
		"and then some..."
		^self].
	layout == #vertical ifTrue:[
		widget layoutPolicy: TableLayout new.
		widget listDirection: #topToBottom.
		"widget cellInset: 1@1.
		widget layoutInset: 1@1."
		widget color: Color transparent.
		"and then some..."
		^self].
	^self error: 'Unknown layout: ', layout.! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'mt 5/3/2015 10:19'!
setLayoutHintsFor: widget spec: aSpec

	aSpec minimumWidth >= 0
		ifTrue: [widget minimumWidth: aSpec minimumWidth].
	aSpec minimumHeight >= 0
		ifTrue: [widget minimumHeight: aSpec minimumHeight].
	aSpec padding
		ifNotNil: [:p | widget layoutInset: p].
	"aSpec margin
		ifNotNil: [:m | widget layoutOutset: m]."
	
	widget hResizing: (aSpec horizontalResizing ifNil: [#spaceFill]).
	widget vResizing: (aSpec verticalResizing ifNil: [#spaceFill]).! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'topa 1/17/2017 17:54'!
setListPropertiesFor: widget spec: aSpec
		
	"Override default scroll bar policies if needed. Widget will use preference values otherwise."
	aSpec hScrollBarPolicy ifNotNil: [:policy |
		policy caseOf: {
			[#always] -> [widget alwaysShowHScrollBar].
			[#never] -> [widget hideHScrollBarIndefinitely].
			[#whenNeeded] -> [widget showHScrollBarOnlyWhenNeeded]. } ].
	aSpec vScrollBarPolicy ifNotNil: [:policy |
		policy caseOf: {
			[#always] -> [widget alwaysShowVScrollBar].
			[#never] -> [widget hideVScrollBarIndefinitely].
			[#whenNeeded] -> [widget showVScrollBarOnlyWhenNeeded]. } ].	
	
	widget
		getListElementSelector: aSpec listItem;
		getListSizeSelector: aSpec listSize;
		getIconSelector: aSpec icon;
		getHelpSelector: aSpec helpItem;
		doubleClickSelector: aSpec doubleClick.
	
	widget
		autoDeselect: aSpec autoDeselect;
		keystrokePreviewSelector: aSpec keystrokePreview.
	
	aSpec filterableList ifNotNil: [:b | widget filterableList: b].
	aSpec clearFilterAutomatically ifNotNil: [:b | widget clearFilterAutomatically: b].
	
	aSpec color ifNotNil: [:c | widget color: c].
	self buildHelpFor: widget spec: aSpec. 
			
	self setFrame: aSpec frame in: widget.
	self setLayoutHintsFor: widget spec: aSpec.
	
	parent ifNotNil:[self add: widget to: parent].
	(panes notNil and: [aSpec list notNil]) ifTrue: [panes add: aSpec list].
! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'mt 5/3/2015 14:49'!
spacerClass

	^ Morph! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'Igor.Stasenko 1/17/2010 04:15'!
textPaneClass
	^ PluggableTextMorphPlus! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'Igor.Stasenko 1/17/2010 04:16'!
treeClass
	^ PluggableTreeMorph! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'ar 7/14/2005 22:30'!
widgetAt: id ifAbsent: aBlock
	widgets ifNil:[^aBlock value].
	^widgets at: id ifAbsent: aBlock! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'Igor.Stasenko 1/17/2010 04:16'!
windowClass
	^ PluggableSystemWindow! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'mt 7/14/2016 07:56'!
windowClassFor: aSpec
	^aSpec multiWindowStyle
		caseOf:
		{	[nil]				->	[PluggableSystemWindow].
			[#labelButton]	->	[PluggableSystemWindowWithLabelButton] }
		otherwise:				[PluggableSystemWindowWithLabelButton]! !

PluggableListMorph subclass: #PluggableListMorphPlus
	instanceVariableNames: 'dragItemSelector dropItemSelector wantsDropSelector dragTypeSelector dragStartedSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggableListMorphPlus commentStamp: 'ar 7/15/2005 11:10' prior: 0!
Extensions for PluggableListMorph needed by ToolBuilder!

PluggableListMorphPlus subclass: #PluggableListMorphByItemPlus
	instanceVariableNames: 'itemList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggableListMorphByItemPlus commentStamp: '<historical>' prior: 0!
Main comment stating the purpose of this class and relevant relationship to other classes.

Possible useful expressions for doIt or printIt.

Structure:
 instVar1		type -- comment about the purpose of instVar1
 instVar2		type -- comment about the purpose of instVar2

Any further useful comments about the general approach of this implementation.!

!PluggableListMorphByItemPlus methodsFor: 'model access' stamp: 'stephaneducasse 2/3/2006 22:35'!
changeModelSelection: anInteger
	"Change the model's selected item to be the one at the given index."

	| item |
	setIndexSelector ifNotNil: [
		item := (anInteger = 0 ifTrue: [nil] ifFalse: [itemList at: anInteger]).
		model perform: setIndexSelector with: item].
	self update: getIndexSelector.
! !

!PluggableListMorphByItemPlus methodsFor: 'model access' stamp: 'jrd 12/8/2008 23:24'!
getCurrentSelectionIndex
	"Answer the index of the current selection."
	| item |
	getIndexSelector == nil ifTrue: [^ 0].
	item := model perform: getIndexSelector.
	^ itemList findFirst: [ :x | x = item]
! !

!PluggableListMorphByItemPlus methodsFor: 'as yet unclassified' stamp: 'ar 7/15/2005 11:23'!
getList
	"cache the raw items in itemList"
	itemList := getListSelector ifNil: [ #() ] ifNotNil: [ model perform: getListSelector ].
	^super getList! !

!PluggableListMorphByItemPlus methodsFor: 'initialization' stamp: 'stephaneducasse 2/3/2006 22:35'!
list: arrayOfStrings
	"Set the receivers items to be the given list of strings."
	"Note: the instance variable 'items' holds the original list.
	 The instance variable 'list' is a paragraph constructed from
	 this list."
"NOTE: this is no longer true; list is a real list, and itemList is no longer used.  And this method shouldn't be called, incidentally."
self isThisEverCalled .
	itemList := arrayOfStrings.
	^ super list: arrayOfStrings! !

!PluggableListMorphPlus methodsFor: 'drag and drop' stamp: 'mt 3/30/2016 15:51'!
acceptDroppingMorph: aTransferMorph event: evt

	dropItemSelector ifNil: [^ self].
	potentialDropRow ifNil: [^ self].
	
	model
		perform: dropItemSelector
		withEnoughArguments: {
			aTransferMorph passenger.
			self modelIndexFor: potentialDropRow.
			aTransferMorph shouldCopy.
			aTransferMorph}.
			
	self resetPotentialDropRow.
	evt hand releaseMouseFocus: self.
	Cursor normal show.
! !

!PluggableListMorphPlus methodsFor: 'accessing' stamp: 'ar 7/15/2005 11:24'!
dragItemSelector
	^dragItemSelector! !

!PluggableListMorphPlus methodsFor: 'accessing' stamp: 'ar 7/15/2005 11:26'!
dragItemSelector: aSymbol
	dragItemSelector := aSymbol.
	aSymbol ifNotNil:[self dragEnabled: true].! !

!PluggableListMorphPlus methodsFor: 'accessing' stamp: 'mt 11/4/2015 14:42'!
dragStartedSelector
	^ dragStartedSelector! !

!PluggableListMorphPlus methodsFor: 'accessing' stamp: 'mt 11/4/2015 14:43'!
dragStartedSelector: symbol
	dragStartedSelector := symbol.! !

!PluggableListMorphPlus methodsFor: 'accessing' stamp: 'mt 6/10/2015 10:18'!
dragTypeSelector
	^dragTypeSelector! !

!PluggableListMorphPlus methodsFor: 'accessing' stamp: 'mt 6/10/2015 10:18'!
dragTypeSelector: aSymbol
	dragTypeSelector := aSymbol.! !

!PluggableListMorphPlus methodsFor: 'accessing' stamp: 'ar 7/15/2005 11:25'!
dropItemSelector
	^dropItemSelector! !

!PluggableListMorphPlus methodsFor: 'accessing' stamp: 'ar 7/15/2005 11:25'!
dropItemSelector: aSymbol
	dropItemSelector := aSymbol.
	aSymbol ifNotNil:[self dropEnabled: true].! !

!PluggableListMorphPlus methodsFor: 'drag and drop' stamp: 'mt 11/5/2015 08:59'!
startDrag: evt 

	| item itemMorph modelIndex |	
	dragItemSelector ifNil:[^self].
	evt hand hasSubmorphs ifTrue: [^ self].
	self model okToChange ifFalse: [^ self].

	"Ensure selection to save additional click."
	(self modelIndexFor: (self rowAtLocation: evt position)) in: [:evtIndex |
		self selectionIndex = evtIndex ifFalse: [self changeModelSelection: evtIndex]].

	item := self selection ifNil: [^ self].
	itemMorph := StringMorph contents: item asStringOrText.
	modelIndex := self modelIndexFor: self selectionIndex.

	[ "Initiate drag."
		(self model perform: dragItemSelector with: modelIndex) ifNotNil: [:passenger | | ddm |
			ddm := (self valueOfProperty: #dragTransferClass ifAbsent: [TransferMorph]) withPassenger: passenger from: self.
			ddm dragTransferType: (self dragTypeSelector ifNotNil: [:s | self model perform: s with: modelIndex]).
			ddm updateFromUserInputEvent: evt.
			self dragStartedSelector ifNotNil: [:s | self model perform: s with: itemMorph with: ddm].
			evt hand grabMorph: ddm].
	] ensure: [
		Cursor normal show.
		evt hand releaseMouseFocus: self]! !

!PluggableListMorphPlus methodsFor: 'accessing' stamp: 'ar 7/15/2005 12:07'!
wantsDropSelector
	^wantsDropSelector! !

!PluggableListMorphPlus methodsFor: 'accessing' stamp: 'ar 7/15/2005 12:07'!
wantsDropSelector: aSymbol
	wantsDropSelector := aSymbol! !

!PluggableListMorphPlus methodsFor: 'drag and drop' stamp: 'mt 11/4/2015 14:11'!
wantsDroppedMorph: aTransferMorph event: anEvent

	dropItemSelector ifNil: [^ false].
	wantsDropSelector ifNil: [^ true].

	(aTransferMorph isKindOf: TransferMorph) ifFalse: [^ false].

	^ model perform: wantsDropSelector withEnoughArguments: {
		aTransferMorph passenger.
		aTransferMorph dragTransferType.
		aTransferMorph source.
		aTransferMorph}! !

!DialogWindow methodsFor: '*ToolBuilder-Morphic-opening' stamp: 'mt 10/5/2017 08:42'!
openAsTool

	self flag: #fishy. "mt: Why does the dialog not return its Morphic form but data?"
	^ self
		moveToHand;
		getUserResponse! !

DialogWindow subclass: #PluggableDialogWindow
	instanceVariableNames: 'model getTitleSelector getMessageSelector getChildrenSelector getButtonsSelector closeDialogSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggableDialogWindow methodsFor: 'accessing' stamp: 'mt 7/14/2016 08:36'!
closeDialogSelector

	^ closeDialogSelector! !

!PluggableDialogWindow methodsFor: 'accessing' stamp: 'mt 7/14/2016 08:36'!
closeDialogSelector: anObject

	closeDialogSelector := anObject! !

!PluggableDialogWindow methodsFor: 'submorphs-add/remove' stamp: 'mt 7/14/2016 08:36'!
delete

	self model okToClose ifFalse: [^ self].
	
	self closeDialogSelector ifNotNil: [:sel | self model perform: sel].
	
	self model
		windowIsClosing;
		release.
	self model: nil.
	
	super delete.! !

!PluggableDialogWindow methodsFor: 'accessing' stamp: 'mt 7/14/2016 08:31'!
getButtonsSelector

	^ getButtonsSelector! !

!PluggableDialogWindow methodsFor: 'accessing' stamp: 'mt 7/14/2016 08:31'!
getButtonsSelector: anObject

	getButtonsSelector := anObject! !

!PluggableDialogWindow methodsFor: 'accessing' stamp: 'mt 7/14/2016 08:31'!
getChildrenSelector

	^ getChildrenSelector! !

!PluggableDialogWindow methodsFor: 'accessing' stamp: 'mt 7/14/2016 08:31'!
getChildrenSelector: anObject

	getChildrenSelector := anObject! !

!PluggableDialogWindow methodsFor: 'accessing' stamp: 'mt 7/14/2016 08:31'!
getMessageSelector

	^ getMessageSelector! !

!PluggableDialogWindow methodsFor: 'accessing' stamp: 'mt 7/14/2016 08:31'!
getMessageSelector: anObject

	getMessageSelector := anObject! !

!PluggableDialogWindow methodsFor: 'accessing' stamp: 'mt 7/14/2016 08:31'!
getTitleSelector

	^ getTitleSelector! !

!PluggableDialogWindow methodsFor: 'accessing' stamp: 'mt 7/14/2016 08:31'!
getTitleSelector: anObject

	getTitleSelector := anObject! !

!PluggableDialogWindow methodsFor: 'accessing' stamp: 'mt 7/14/2016 08:17'!
model
	^ model! !

!PluggableDialogWindow methodsFor: 'accessing' stamp: 'mt 7/14/2016 08:18'!
model: anObject

	model ifNotNil: [model removeDependent: self].
	anObject ifNotNil: [anObject addDependent: self].
	model := anObject.! !

!PluggableDialogWindow methodsFor: 'updating' stamp: 'mt 7/14/2016 17:01'!
update: what

	what ifNil:[^self].

	what == self getTitleSelector ifTrue:[self title: (model perform: self getTitleSelector)].
	what == self getMessageSelector ifTrue:[self message: (model perform: self getMessageSelector)].
	
	what == self getChildrenSelector ifTrue:[
		self paneMorph removeAllMorphs.
		(self model perform: self getChildrenSelector)
			do: [:m| m hResizing: #spaceFill; vResizing: #spaceFill];
			in: [:children | self paneMorph addAllMorphs: children]].

	what == self getButtonsSelector ifTrue:[
		self buttonRow
			removeAllMorphs;
			addAllMorphs: (self model perform: self getButtonsSelector).
		self updateButtonProperties].
	
	what == #close ifTrue: [^ self delete].

	super update: what.! !

!PluggableDialogWindow methodsFor: 'updating' stamp: 'mt 7/21/2016 09:16'!
updateButtonProperties

	self buttons do: [:ea |
		ea setProperty: #normalColor toValue: ea offColor.
		ea setProperty: #normalLabel toValue: ea label.
		ea hResizing: #rigid; vResizing: #rigid].
	
	self updateButtonExtent.! !

AlignmentMorph subclass: #PluggableCheckBoxMorph
	instanceVariableNames: 'model actionSelector valueSelector label'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggableCheckBoxMorph class methodsFor: 'as yet unclassified' stamp: 'rww 9/11/2010 03:44'!
on: anObject getState: getStateSel action: actionSel label: labelSel

	^ self new
		on: anObject
		getState: getStateSel
		action: actionSel
		label: labelSel
		menu: nil
! !

!PluggableCheckBoxMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 03:37'!
actionSelector
	"Answer the value of actionSelector"

	^ actionSelector! !

!PluggableCheckBoxMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 03:37'!
actionSelector: anObject
	"Set the value of actionSelector"

	actionSelector := anObject! !

!PluggableCheckBoxMorph methodsFor: 'installing' stamp: 'rww 9/11/2010 04:10'!
basicPanel
	^BorderedMorph new
		beTransparent;
		extent: 0@0;
		borderWidth: 0;
		layoutInset: 0;
		cellInset: 0;
		layoutPolicy: TableLayout new;
		listCentering: #topLeft;
		cellPositioning: #center;
		hResizing: #spaceFill;
		vResizing: #shrinkWrap;
		yourself! !

!PluggableCheckBoxMorph methodsFor: 'installing' stamp: 'rww 9/11/2010 04:10'!
horizontalPanel
	^self basicPanel
		cellPositioning: #center;
		listDirection: #leftToRight;
		yourself.! !

!PluggableCheckBoxMorph methodsFor: 'installing' stamp: 'topa 7/12/2016 00:17'!
installButton

	| aButton aLabel |
	aButton := UpdatingThreePhaseButtonMorph checkBox
		target: self model;
		actionSelector: self actionSelector;
		getSelector: self valueSelector;
		yourself.
	aLabel := (StringMorph contents: self label translated
				font: (StrikeFont familyName: TextStyle defaultFont familyName
							pointSize: TextStyle defaultFont pointSize - 1)).
	self addMorph: (self horizontalPanel
		addMorphBack: aButton;
		addMorphBack: aLabel;
		yourself).! !

!PluggableCheckBoxMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 03:37'!
label
	"Answer the value of label"

	^ label! !

!PluggableCheckBoxMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 03:37'!
label: anObject
	"Set the value of label"

	label := anObject! !

!PluggableCheckBoxMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 03:38'!
model
	"Answer the value of model"

	^ model.
! !

!PluggableCheckBoxMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 03:37'!
model: anObject
	"Set the value of model"

	model := anObject! !

!PluggableCheckBoxMorph methodsFor: 'initialization' stamp: 'rww 9/11/2010 03:45'!
on: anObject getState: getStateSel action: actionSel label: labelSel menu: menuSel

	self model: anObject.
	self valueSelector: getStateSel.
	self actionSelector: actionSel.
	self label: (self model perform: labelSel).
! !

!PluggableCheckBoxMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 03:37'!
valueSelector
	"Answer the value of valueSelector"

	^ valueSelector! !

!PluggableCheckBoxMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 03:37'!
valueSelector: anObject
	"Set the value of valueSelector"

	valueSelector := anObject! !

AlignmentMorph subclass: #PluggableDropDownListMorph
	instanceVariableNames: 'model listSelector selectionSelector selectionSetter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggableDropDownListMorph methodsFor: 'drawing' stamp: 'rww 9/11/2010 14:22'!
basicPanel
	^BorderedMorph new
		beTransparent;
		extent: 0@0;
		borderWidth: 0;
		layoutInset: 0;
		cellInset: 0;
		layoutPolicy: TableLayout new;
		listCentering: #topLeft;
		cellPositioning: #center;
		hResizing: #spaceFill;
		vResizing: #shrinkWrap;
		yourself! !

!PluggableDropDownListMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 13:34'!
currentSelection

	^ self model perform: selectionSelector! !

!PluggableDropDownListMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 13:34'!
currentSelection: obj

	^ self model perform: selectionSetter with: obj! !

!PluggableDropDownListMorph methodsFor: 'drawing' stamp: 'rww 9/11/2010 14:22'!
horizontalPanel
	^self basicPanel
		cellPositioning: #center;
		listDirection: #leftToRight;
		yourself.! !

!PluggableDropDownListMorph methodsFor: 'drawing' stamp: 'topa 7/12/2016 00:18'!
installDropDownList

	| aButton aLabel |
	aButton := PluggableButtonMorph on: self model getState: nil action: nil.
	aLabel := (StringMorph contents: self model currentRemoteVatId translated
				font: (StrikeFont familyName: TextStyle defaultFont familyName
							pointSize: TextStyle defaultFont pointSize - 1)).
	self addMorph: (self horizontalPanel
		addMorphBack: aLabel;
		addMorphBack: aButton;
		yourself).! !

!PluggableDropDownListMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 13:33'!
list
	"Answer the value of list"

	^ self model perform: self listSelector.
	! !

!PluggableDropDownListMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 13:32'!
listSelector
	"Answer the value of listSelector"

	^ listSelector! !

!PluggableDropDownListMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 13:32'!
listSelector: anObject
	"Set the value of listSelector"

	listSelector := anObject! !

!PluggableDropDownListMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 13:33'!
model
	^ model! !

!PluggableDropDownListMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 13:32'!
model: anObject
	"Set the value of model"

	model := anObject! !

!PluggableDropDownListMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 13:32'!
selectionSelector
	"Answer the value of selectionSelector"

	^ selectionSelector! !

!PluggableDropDownListMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 13:32'!
selectionSelector: anObject
	"Set the value of selectionSelector"

	selectionSelector := anObject! !

!PluggableDropDownListMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 13:32'!
selectionSetter
	"Answer the value of selectionSetter"

	^ selectionSetter! !

!PluggableDropDownListMorph methodsFor: 'accessing' stamp: 'rww 9/11/2010 13:32'!
selectionSetter: anObject
	"Set the value of selectionSetter"

	selectionSetter := anObject! !

AlignmentMorph subclass: #PluggablePanelMorph
	instanceVariableNames: 'model getChildrenSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggablePanelMorph commentStamp: 'ar 2/11/2005 20:13' prior: 0!
A pluggable panel morph which deals with changing children.!

!PluggablePanelMorph methodsFor: 'private' stamp: 'cmm 8/25/2013 13:09'!
canBeEncroached
	^ submorphs allSatisfy:
		[ : each | each canBeEncroached ]! !

!PluggablePanelMorph methodsFor: 'accessing' stamp: 'cmm 8/22/2013 21:50'!
children
	^ model perform: getChildrenSelector! !

!PluggablePanelMorph methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:47'!
getChildrenSelector
	^getChildrenSelector! !

!PluggablePanelMorph methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:47'!
getChildrenSelector: aSymbol
	getChildrenSelector := aSymbol.! !

!PluggablePanelMorph methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:48'!
model
	^model! !

!PluggablePanelMorph methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:48'!
model: aModel
	model ifNotNil:[model removeDependent: self].
	model := aModel.
	model ifNotNil:[model addDependent: self].! !

!PluggablePanelMorph methodsFor: 'update' stamp: 'mt 6/5/2016 13:17'!
update: selectorSymbolOrNil 
	selectorSymbolOrNil ifNil: [ ^ self ].
	selectorSymbolOrNil = getChildrenSelector ifTrue:
		[ self
			 removeAllMorphs ;
			 addAllMorphs: self children .
			
		"Let my children take all the place unless my extent is described by them."
		self hResizing ~~ #shrinkWrap ifTrue:
			[ self submorphsDo: [ : m | m hResizing: #spaceFill ] ].
		self vResizing ~~ #shrinkWrap ifTrue:
			[ self submorphsDo: [ : m | m vResizing: #spaceFill ] ].
		
		"Tell dependents about this update. There is no pluggable notification for changed submorphs."
		self changed: #submorphs ].! !

Model subclass: #ListChooser
	instanceVariableNames: 'selectedIndex items searchText addAllowed result title listMorph dialogMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!ListChooser commentStamp: 'MAD 3/14/2010 16:20' prior: 0!
I am a simple dialog to allow the user to pick from a list of strings or symbols.
I support keyboard and mouse navigation, and interactive filtering of the displayed items.

You can specify whether you want the index, or the value of the selected item. If you're interested in the value, you can also allow users to Add a new value not in the list.

cmd-s or <enter> or double-click answers the currently selected item's value/index;
cmd-l or <escape> or closing the window answers nil/zero.

Now using ToolBuilder, so needs Morphic-MAD.381.

Released under the MIT Licence.!

!ListChooser class methodsFor: 'ChooserTool compatibility' stamp: 'MAD 3/14/2010 15:11'!
chooseFrom: aList
	^ self 
		chooseFrom: aList 
		title: self defaultTitle! !

!ListChooser class methodsFor: 'ChooserTool compatibility' stamp: 'MAD 3/14/2010 15:31'!
chooseFrom: aList title: aString
	^ self
		chooseIndexFrom: aList 
		title: aString
		addAllowed: false! !

!ListChooser class methodsFor: 'instance creation' stamp: 'MAD 3/14/2010 15:26'!
chooseIndexFrom: aList
	^ self 
		chooseIndexFrom: aList 
		title: self defaultTitle! !

!ListChooser class methodsFor: 'instance creation' stamp: 'cmm 3/15/2010 17:19'!
chooseIndexFrom: aList title: aString
	^ self
		chooseIndexFrom: aList 
		title: (aString isEmptyOrNil ifTrue: [ self defaultTitle ] ifFalse: [ aString ])
		addAllowed: false! !

!ListChooser class methodsFor: 'instance creation' stamp: 'cmm 3/15/2010 17:18'!
chooseIndexFrom: aList title: aString addAllowed: aBoolean
	^ self new
		chooseIndexFrom: aList 
		title: (aString isEmptyOrNil ifTrue: [ self defaultTitle ] ifFalse: [ aString ])
		addAllowed: aBoolean! !

!ListChooser class methodsFor: 'instance creation' stamp: 'MAD 3/14/2010 15:24'!
chooseItemFrom: aList
	^ self 
		chooseItemFrom: aList 
		title: self defaultTitle! !

!ListChooser class methodsFor: 'instance creation' stamp: 'MAD 3/14/2010 15:25'!
chooseItemFrom: aList title: aString
	^ self
		chooseItemFrom: aList 
		title: aString
		addAllowed: false! !

!ListChooser class methodsFor: 'instance creation' stamp: 'MAD 3/14/2010 15:25'!
chooseItemFrom: aList title: aString addAllowed: aBoolean
	^ self new
		chooseItemFrom: aList 
		title: (aString isEmptyOrNil ifTrue: [ self defaultTitle ] ifFalse: [ aString ])
		addAllowed: aBoolean! !

!ListChooser class methodsFor: 'instance creation' stamp: 'MAD 3/13/2010 18:56'!
defaultTitle
	^ 'Please choose:'! !

!ListChooser class methodsFor: 'examples' stamp: 'MAD 3/14/2010 15:37'!
testDictionary
	^ self 
		chooseItemFrom: (Dictionary newFrom: {#a->1. 2->#b.})
		title: 'Pick from Dictionary' "gives values, not keys"! !

!ListChooser class methodsFor: 'examples' stamp: 'MAD 3/14/2010 15:27'!
testIndex
	^ self 
		chooseIndexFrom: (Smalltalk classNames , Smalltalk traitNames) asOrderedCollection
		title: 'Pick a class'! !

!ListChooser class methodsFor: 'examples' stamp: 'MAD 3/14/2010 15:29'!
testItem
	^ self 
		chooseItemFrom: (Smalltalk classNames , Smalltalk traitNames) asOrderedCollection
		title: 'Pick a class'! !

!ListChooser class methodsFor: 'examples' stamp: 'MAD 3/14/2010 15:31'!
testItemAdd
	^ self 
		chooseItemFrom: (Smalltalk classNames , Smalltalk traitNames) asOrderedCollection
		title: 'Pick or Add:'
		addAllowed: true! !

!ListChooser class methodsFor: 'examples' stamp: 'MAD 3/15/2010 16:08'!
testLongTitle
	^ self 
		chooseItemFrom: #(this is a list of values that aren/t the point here)
		title: 'Pick from some values from this list'! !

!ListChooser class methodsFor: 'examples' stamp: 'MAD 3/14/2010 15:33'!
testSet
	^ self 
		chooseItemFrom: #(a list of values as a Set) asSet
		title: 'Pick from Set'! !

!ListChooser class methodsFor: 'preferences' stamp: 'mt 7/10/2016 20:02'!
themeProperties

	^ super themeProperties,  {
		{ #okColor. 'Colors'. 'Color for the OK button.' }.
		{ #cancelColor. 'Colors'. 'Color for the Cancel button.' }.
		{ #addColor. 'Colors'. 'Color for a normal button.' }.
		{ #disabledColor. 'Colors'. 'Color for a disabled button.' }.
	}! !

!ListChooser methodsFor: 'actions' stamp: 'mt 7/22/2016 10:40'!
accept
	"if the user submits with no valid entry, make them start over"
	
	| choice |
	self canAccept ifFalse: [
		self canAdd ifTrue: [^ self add].
		^ self changed: #textSelection].
	
	choice := self selectedItem.
	
	self canAdd ifTrue: [
		"Ask the user whether to add the new item or choose the list selection."
		(UserDialogBoxMorph confirm: 'You can either choose an existing item or add a new one.\What do you want?' translated withCRs title: 'Choose or Add' translated trueChoice: choice asString falseChoice: self searchText asString at: ActiveHand position)
			ifTrue: [self result: choice] ifFalse: [self result: self searchText asString]
		] ifFalse: [self result: choice].
		
	
	self changed: #close.! !

!ListChooser methodsFor: 'colors' stamp: 'mt 7/22/2016 15:50'!
acceptColor
	
	self canAdd ifTrue: [^ self addColor].
	
	^ self canAccept 
		ifTrue: [ self userInterfaceTheme okColor ifNil: [(Color r: 0.49 g: 0.749 b: 0.49)] ]
		ifFalse: [ self userInterfaceTheme disabledColor ifNil: [Color lightGray] ]! !

!ListChooser methodsFor: 'colors' stamp: 'mt 7/20/2016 13:59'!
acceptLabel

	^ self canAdd
		ifFalse: ['Choose' translated]
		ifTrue: [self canAccept
			ifTrue: ['Choose or Add' translated]
			ifFalse: ['Add' translated]]! !

!ListChooser methodsFor: 'actions' stamp: 'MAD 3/13/2010 18:56'!
acceptText: someText
	"the text morph wants to tell us about its contents but I don't care, I'm only interested in the list"
	self accept! !

!ListChooser methodsFor: 'actions' stamp: 'mt 7/19/2016 11:33'!
add
	"if the user submits with no valid entry, make them start over"
	self canAdd ifFalse: [^ self changed: #textSelection].
	self result: self searchText asString.
	self changed: #close.! !

!ListChooser methodsFor: 'accessing' stamp: 'mt 8/5/2016 09:57'!
addAllowed

	^ addAllowed ifNil: [false]! !

!ListChooser methodsFor: 'accessing' stamp: 'mt 7/14/2016 17:19'!
addAllowed: anObject

	addAllowed := anObject! !

!ListChooser methodsFor: 'colors' stamp: 'mt 7/10/2016 20:01'!
addColor
	
	^ self canAdd 
		ifTrue: [ self userInterfaceTheme addColor ifNil: [Color blue muchLighter] ]
		ifFalse: [ self userInterfaceTheme disabledColor ifNil: [Color lightGray] ]! !

!ListChooser methodsFor: 'updating' stamp: 'mt 7/21/2016 09:23'!
applyUserInterfaceTheme

	super applyUserInterfaceTheme.
	
	self
		changed: #okColor;
		changed: #cancelColor;
		changed: #addColor.! !

!ListChooser methodsFor: 'building' stamp: 'mt 8/19/2016 18:32'!
buildWith: builder

	| dialogSpec searchBarHeight listSpec fieldSpec |
	
	searchBarHeight := Preferences standardDefaultTextFont height * 1.75.
	
	dialogSpec := builder pluggableDialogSpec new
		model: self;
		title: #title;
		closeAction: #closed;
		extent: self initialExtent;
		autoCancel: true; "behave like a pop-up menu"
		children: OrderedCollection new;
		buttons: OrderedCollection new;
		yourself.
	
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #items; 
		getIndex: #selectedIndex; 
		setIndex: #selectedIndex:; 
		doubleClick: #accept;
		"keystrokePreview: #keyStrokeFromList:;"
		autoDeselect: false;
		filterableList: true;
		clearFilterAutomatically: false;
		name: #list;
		frame: (LayoutFrame fractions: (0@0 corner: 1@1) offsets: (0@searchBarHeight corner: 0@0)).
	dialogSpec children add: listSpec.
	
	fieldSpec := builder pluggableInputFieldSpec new.
	fieldSpec 
		model: self;
		getText: #searchText;
		editText: #searchText:;
		setText: #acceptText:;
		selection: #textSelection;
		menu: nil;
		indicateUnacceptedChanges: false;
		askBeforeDiscardingEdits: false;
		help: (self addAllowed ifTrue: ['Type new or filter existing...' translated] ifFalse: ['Type to filter existing...' translated]);
		frame: (LayoutFrame fractions: (0@0 corner: 1@0) offsets: (0@0 corner: 0@searchBarHeight)).
	dialogSpec children add: fieldSpec.
	
	"Buttons"
	dialogSpec buttons add: (
		builder pluggableButtonSpec new
			model: self; 
			label: #acceptLabel;
			action: #accept;
			enabled: #canAcceptOrAdd;
			color: #acceptColor).

	dialogSpec buttons add: (
		builder pluggableButtonSpec new
			model: self; 
			label: 'Cancel';
			action: #cancel;
			color: #cancelColor).
		
	dialogMorph := builder build: dialogSpec.
	dialogMorph addKeyboardCaptureFilter: self.
	listMorph := builder widgetAt: #list.
	listMorph allowEmptyFilterResult: true.
	
	^ dialogMorph! !

!ListChooser methodsFor: 'testing' stamp: 'MAD 3/13/2010 18:56'!
canAccept
	^ self selectedIndex > 0! !

!ListChooser methodsFor: 'testing' stamp: 'mt 7/19/2016 12:12'!
canAcceptOrAdd
	^ self canAccept or: [self canAdd]! !

!ListChooser methodsFor: 'testing' stamp: 'mt 7/22/2016 10:41'!
canAdd
	^ self addAllowed
		and: [self searchText asString withBlanksTrimmed notEmpty]
		and: [self selectedItem asString ~= self searchText asString]! !

!ListChooser methodsFor: 'actions' stamp: 'mt 7/19/2016 11:33'!
cancel

	self result: nil.
	self changed: #close.! !

!ListChooser methodsFor: 'colors' stamp: 'mt 7/22/2016 15:51'!
cancelColor
	
	^ self userInterfaceTheme cancelColor ifNil: [Color r: 1 g: 0.6 b: 0.588]! !

!ListChooser methodsFor: 'initialize-release' stamp: 'ul 2/27/2017 00:28'!
chooseIndexFrom: labelList title: aString
	| choice |
	choice := self chooseItemFrom: labelList title: aString addAllowed: false.
	^ self items indexOf: choice! !

!ListChooser methodsFor: 'initialize-release' stamp: 'ul 2/27/2017 00:28'!
chooseIndexFrom: labelList title: aString addAllowed: aBoolean
	| choice |
	choice := self chooseItemFrom: labelList title: aString addAllowed: false.
	self addAllowed: aBoolean.
	^ self items indexOf: choice! !

!ListChooser methodsFor: 'initialize-release' stamp: 'mt 7/14/2016 17:21'!
chooseItemFrom: labelList title: aString addAllowed: aBoolean

	self items: labelList asOrderedCollection.
	self title: aString.
	self addAllowed: aBoolean.

	ToolBuilder open: self.
	^ self result! !

!ListChooser methodsFor: 'actions' stamp: 'mt 7/14/2016 17:23'!
closed
	
	self selectedIndex: 0.! !

!ListChooser methodsFor: 'event handling' stamp: 'nice 10/28/2016 00:49'!
filterEvent: aKeyboardEvent for: aMorph

	| char |
	aKeyboardEvent isKeystroke ifFalse: [^ aKeyboardEvent].
	aKeyboardEvent anyModifierKeyPressed ifTrue: [^ aKeyboardEvent].
	
	char := aKeyboardEvent keyCharacter.
	
	char = Character backspace
		ifTrue: [self searchText: (self searchText asString ifNotEmpty: [:s | s allButLast]). ^ aKeyboardEvent ignore].
	char = Character delete
		ifTrue: [self searchText: (self searchText asString ifNotEmpty: [:s | s allButFirst]). ^ aKeyboardEvent ignore].
	(char = Character cr or: [char = Character enter])
		ifTrue: [self accept. aKeyboardEvent ignore].
	char = Character escape 
		ifTrue: [self cancel. aKeyboardEvent ignore].
	(char asInteger between: 32 and: 126)
		ifTrue: [self searchText: self searchText asString, char asString. aKeyboardEvent ignore].
	(char = Character arrowUp or: [char = Character arrowDown
		or: [char = Character pageUp or: [char = Character pageDown
		or: [char = Character home or: [char = Character end]]]]])
		ifTrue: [listMorph keyStroke: aKeyboardEvent. ^aKeyboardEvent ignore].
	
	^ aKeyboardEvent! !

!ListChooser methodsFor: 'building' stamp: 'mt 7/14/2016 17:04'!
initialExtent

	| listFont |
	listFont := Preferences standardListFont.
	^ (20 * (listFont widthOf: $m))@(15 * listFont height)! !

!ListChooser methodsFor: 'accessing' stamp: 'mt 7/14/2016 17:19'!
items

	^ items! !

!ListChooser methodsFor: 'accessing' stamp: 'mt 7/14/2016 17:19'!
items: anObject

	items := anObject! !

!ListChooser methodsFor: 'accessing' stamp: 'mt 7/14/2016 17:19'!
result

	^ result! !

!ListChooser methodsFor: 'accessing' stamp: 'mt 7/14/2016 17:19'!
result: anObject

	result := anObject! !

!ListChooser methodsFor: 'accessing' stamp: 'MAD 3/13/2010 18:56'!
searchText
	^ searchText ifNil: [ searchText := '' ]! !

!ListChooser methodsFor: 'accessing' stamp: 'mt 7/21/2016 09:18'!
searchText: aString
	searchText := aString.
	listMorph filterList: aString asString.
	
	self changed: #searchText.
	self changed: #canAcceptOrAdd.
	self changed: #acceptLabel.
	self changed: #buttons.! !

!ListChooser methodsFor: 'accessing' stamp: 'mt 8/5/2016 09:29'!
selectedIndex
	^ selectedIndex ifNil: [ self items size min: 1 ]! !

!ListChooser methodsFor: 'accessing' stamp: 'mt 7/21/2016 09:18'!
selectedIndex: anInt
	selectedIndex := anInt.
	self changed: #selectedIndex.! !

!ListChooser methodsFor: 'accessing' stamp: 'mt 7/22/2016 10:40'!
selectedItem

	^  self items at: self selectedIndex ifAbsent: []! !

!ListChooser methodsFor: 'accessing' stamp: 'mt 7/19/2016 12:00'!
textSelection
	^ self searchText size +1 to: self searchText size ! !

!ListChooser methodsFor: 'accessing' stamp: 'MAD 3/13/2010 18:56'!
title
	^ title ifNil: [ title := 'Please choose' ]! !

!ListChooser methodsFor: 'accessing' stamp: 'MAD 3/13/2010 18:56'!
title: aString
	title := aString.! !

Model subclass: #ListMultipleChooser
	instanceVariableNames: 'selection labels values title choice'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!ListMultipleChooser commentStamp: 'mt 4/14/2015 17:09' prior: 0!
I am like the ListChooser but for multiple choices. I have no extra search field. Enable the preference #filterableLists if lists get too big to choose from. Also, I do not support adding new items to the choice.!

!ListMultipleChooser class methodsFor: 'ui requests' stamp: 'mt 4/14/2015 17:23'!
chooseFrom: someLabels title: aString

	^ self chooseIndexListFrom: someLabels title: aString! !

!ListMultipleChooser class methodsFor: 'ui requests' stamp: 'mt 4/14/2015 15:51'!
chooseIndexListFrom: someObjects title: aString

	^ self new
		title: aString;
		labels: (someObjects collect: [:ea | ea asString]);
		choose! !

!ListMultipleChooser class methodsFor: 'ui requests' stamp: 'mt 4/14/2015 15:51'!
chooseItemListFrom: someObjects title: aString

	^ self new
		title: aString;
		labels: (someObjects collect: [:ea | ea asString]);
		values: someObjects;
		choose! !

!ListMultipleChooser class methodsFor: 'preferences' stamp: 'mt 7/10/2016 20:16'!
themeProperties

	^ super themeProperties,  {
		{ #okColor. 'Colors'. 'Color for the OK button.' }.
		{ #cancelColor. 'Colors'. 'Color for the Cancel button.' }.
	}! !

!ListMultipleChooser methodsFor: 'actions' stamp: 'mt 4/14/2015 15:29'!
accept

	choice := #accepted.
	self changed: #close.! !

!ListMultipleChooser methodsFor: 'updating' stamp: 'mt 7/10/2016 20:15'!
applyUserInterfaceTheme

	super applyUserInterfaceTheme.
	
	self
		changed: #okColor;
		changed: #cancelColor.! !

!ListMultipleChooser methodsFor: 'toolbuilder' stamp: 'mt 7/22/2016 15:52'!
buildWith: builder

	| dialogSpec choicesSpec |
	dialogSpec := builder pluggableDialogSpec new
		model: self;
		extent: self initialExtent;
		title: #title;
		children: OrderedCollection new;
		buttons: OrderedCollection new.
		
	choicesSpec := builder pluggableMultiSelectionListSpec new
		model: self;
		list: #labels;
		setIndex: #selectedIndex:;
		getIndex: #selectedIndex;
		setSelectionList: #selectionAt:put:;
		getSelectionList: #selectionAt:;
		frame: (0@0 corner: 1@1).
	dialogSpec children add: choicesSpec.
	
	"Buttons"
	dialogSpec buttons add: (
		builder pluggableButtonSpec new
			model: self;
			label: 'accept';
			color: (self userInterfaceTheme okColor ifNil: [Color r: 0.49 g: 0.749 b: 0.49]);
			action: #accept).
			
	dialogSpec buttons add: (
		builder pluggableButtonSpec new
			model: self;
			label: 'cancel';
			color: (self userInterfaceTheme cancelColor ifNil: [Color r: 1 g: 0.6 b: 0.588]);
			action: #cancel).
				
	^ builder build: dialogSpec! !

!ListMultipleChooser methodsFor: 'actions' stamp: 'mt 4/14/2015 15:29'!
cancel

	choice := #cancelled.
	self changed: #close.! !

!ListMultipleChooser methodsFor: 'actions' stamp: 'mt 7/21/2016 10:31'!
choose

	
"	self changed: #inputRequested with: #selectedIndex.
"	
	ToolBuilder open: self.
	^ self selectedValues! !

!ListMultipleChooser methodsFor: 'toolbuilder' stamp: 'mt 7/21/2016 10:33'!
initialExtent

	| listFont |
	listFont := Preferences standardListFont.
	^ (20 * (listFont widthOf: $m))@(15 * listFont height)! !

!ListMultipleChooser methodsFor: 'accessing' stamp: 'mt 4/14/2015 15:11'!
labels

	^ labels! !

!ListMultipleChooser methodsFor: 'accessing' stamp: 'mt 4/14/2015 15:12'!
labels: someStrings

	labels := someStrings.
	self changed: #labels.! !

!ListMultipleChooser methodsFor: 'accessing' stamp: 'mt 4/14/2015 15:12'!
selectedIndex
	"Sigh. Required from widget..."
	^ 0! !

!ListMultipleChooser methodsFor: 'accessing' stamp: 'mt 4/14/2015 15:12'!
selectedIndex: anIndex
	"Sigh. Required from widget..."
	self changed: #selectedIndex.! !

!ListMultipleChooser methodsFor: 'accessing' stamp: 'mt 4/14/2015 15:30'!
selectedValues
	
	| i |
	choice ~~ #accepted ifTrue: [^ nil].
	
	i := 0.
	^ self values select: [:object | i := i + 1. self selection at: i]! !

!ListMultipleChooser methodsFor: 'accessing' stamp: 'mt 4/14/2015 15:14'!
selection

	^ selection ifNil: [selection := Array new: self values size withAll: false]! !

!ListMultipleChooser methodsFor: 'accessing' stamp: 'mt 4/14/2015 15:55'!
selectionAt: index

	^ self selection at: index! !

!ListMultipleChooser methodsFor: 'accessing' stamp: 'mt 4/14/2015 15:13'!
selectionAt: index put: boolean

	self selection at: index put: boolean.
	self changed: #selectedIndex! !

!ListMultipleChooser methodsFor: 'accessing' stamp: 'mt 4/14/2015 15:58'!
title

	^ (title isNil or: [title isEmpty])
		ifFalse: [title]
		ifTrue: ['Choose multiple:']! !

!ListMultipleChooser methodsFor: 'accessing' stamp: 'mt 4/14/2015 15:27'!
title: aString

	title := aString.
	self changed: #title.! !

!ListMultipleChooser methodsFor: 'accessing' stamp: 'mt 4/14/2015 15:14'!
values

	^ values ifNil: [values := (1 to: self labels size) asArray]! !

!ListMultipleChooser methodsFor: 'accessing' stamp: 'mt 4/14/2015 15:11'!
values: someObjects

	values := someObjects.! !

!MenuMorph methodsFor: '*ToolBuilder-Morphic-opening' stamp: 'mt 10/5/2017 08:39'!
openAsTool

	self popUpInWorld: World.! !

PluggableButtonMorph subclass: #PluggableButtonMorphPlus
	instanceVariableNames: 'enabled action getColorSelector getEnabledSelector updateMap disabledColor disabledTextColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggableButtonMorphPlus commentStamp: 'ar 2/11/2005 21:53' prior: 0!
An extended version of PluggableButtonMorph supporting enablement, color and block/message actions.!

!PluggableButtonMorphPlus class methodsFor: 'preferences' stamp: 'mt 7/26/2016 10:49'!
themeProperties

	^ super themeProperties, {
		{ #disabledColor. 'Colors'. 'Color when button cannot be clicked.' }.
		{ #disabledTextColor. 'Colors'. 'Color for label when button cannot be clicked.' }.
	}! !

!PluggableButtonMorphPlus methodsFor: 'accessing' stamp: 'ar 2/11/2005 20:53'!
action
	^action! !

!PluggableButtonMorphPlus methodsFor: 'accessing' stamp: 'ar 2/11/2005 20:55'!
action: anAction	
	action := nil.
	anAction isSymbol ifTrue:[^super action: anAction].
	action := anAction.! !

!PluggableButtonMorphPlus methodsFor: 'updating' stamp: 'mt 7/22/2016 15:29'!
applyUserInterfaceTheme

	super applyUserInterfaceTheme.! !

!PluggableButtonMorphPlus methodsFor: 'accessing' stamp: 'dc 1/9/2008 18:17'!
enabled
	^ enabled ifNil: [enabled := true]! !

!PluggableButtonMorphPlus methodsFor: 'accessing' stamp: 'mt 7/22/2016 14:59'!
enabled: aBool
	enabled := aBool.
	self updateFillStyle.! !

!PluggableButtonMorphPlus methodsFor: 'accessing' stamp: 'ar 2/11/2005 21:36'!
getColorSelector
	^getColorSelector! !

!PluggableButtonMorphPlus methodsFor: 'accessing' stamp: 'ar 2/11/2005 21:36'!
getColorSelector: aSymbol
	getColorSelector := aSymbol.
	self update: getColorSelector.! !

!PluggableButtonMorphPlus methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:51'!
getEnabledSelector
	^getEnabledSelector! !

!PluggableButtonMorphPlus methodsFor: 'accessing' stamp: 'mt 7/23/2016 10:51'!
getEnabledSelector: aSymbol
	getEnabledSelector := aSymbol.
	self update: getEnabledSelector.! !

!PluggableButtonMorphPlus methodsFor: 'event handling' stamp: 'mt 7/23/2016 10:27'!
handlesMouseDown: evt
	^ enabled! !

!PluggableButtonMorphPlus methodsFor: 'event handling' stamp: 'mt 7/23/2016 10:28'!
handlesMouseOver: evt
	^ enabled! !

!PluggableButtonMorphPlus methodsFor: 'event handling' stamp: 'mt 7/23/2016 10:28'!
handlesMouseOverDragging: evt
	^ enabled! !

!PluggableButtonMorphPlus methodsFor: 'initialize-release' stamp: 'mt 7/23/2016 10:54'!
initialize

	enabled := true.
	super initialize.! !

!PluggableButtonMorphPlus methodsFor: 'action' stamp: 'ar 2/11/2005 20:54'!
performAction
	enabled ifFalse:[^self].
	action ifNotNil:[^action value].
	^super performAction! !

!PluggableButtonMorphPlus methodsFor: 'initialize-release' stamp: 'mt 7/31/2016 09:29'!
setDefaultParameters

	disabledColor := (self userInterfaceTheme disabledColor ifNil: [Color transparent]).
	disabledTextColor := (self userInterfaceTheme disabledTextColor ifNil: [Color gray: 0.6]).

	super setDefaultParameters.	
	! !

!PluggableButtonMorphPlus methodsFor: 'drawing' stamp: 'mt 7/26/2016 10:55'!
textColorToUse

	^ self enabled
		ifTrue: [super textColorToUse]
		ifFalse: [disabledTextColor ifNil: [Color gray: 0.6]]! !

!PluggableButtonMorphPlus methodsFor: 'updating' stamp: 'nice 6/10/2017 17:35'!
update: what

	super update: what.
		
	getColorSelector ifNotNil: [:sel |
		what == sel ifTrue: [self offColor: (model perform: sel)]].

	getEnabledSelector ifNotNil: [:sel |
		what == sel ifTrue: [self enabled: (model perform: sel)]].

	updateMap ifNotNil:
		[(updateMap at: what ifAbsent: [])
			ifNotNil: [ :newTarget | self update: newTarget]].
! !

!PluggableButtonMorphPlus methodsFor: 'initialize-release' stamp: 'mt 8/3/2016 15:06'!
updateFillStylePressing: isPressing hovering: isHovering

	enabled ifFalse: [
		self color: (disabledColor ifNil: [Color transparent]).
		self borderStyle color: disabledTextColor.
		^ self].
	
	super updateFillStylePressing: isPressing hovering: isHovering.! !

!PluggableButtonMorphPlus methodsFor: 'updating' stamp: 'dtl 9/24/2011 08:54'!
updateMap
	^ updateMap ifNil: [updateMap := Dictionary new]
! !

!PluggableButtonMorphPlus methodsFor: 'updating' stamp: 'dtl 9/24/2011 08:54'!
whenChanged: notification update: target
	"On receipt of a notification, such as #contents notification from a CodeHolder,
	invoke an update as if target had been the original notification."

	self updateMap at: notification put: target! !

PluggableTextMorph subclass: #PluggableTextMorphPlus
	instanceVariableNames: 'getColorSelector acceptAction unstyledAcceptText styler getTextColorSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggableTextMorphPlus commentStamp: 'ar 2/11/2005 21:53' prior: 0!
A pluggable text morph with support for color.!

!PluggableTextMorphPlus class methodsFor: 'preferences' stamp: 'mt 7/25/2016 11:07'!
themeProperties

	^  super themeProperties, {
		{ #stylerClass. 'Editing'. 'The styler class you want to use for syntax highlighting' }.
		{ #balloonTextColor. 'Colors'. 'Color to use when drawing the balloon text on an empty text field.' }.
		{ #balloonTextFont. 'Fonts'. 'Font to use when drawing the balloon text on an empty text field.' }.
	}! !

!PluggableTextMorphPlus methodsFor: 'updating' stamp: 'ar 7/16/2005 19:02'!
accept
	super accept.
	acceptAction ifNotNil:[acceptAction value: textMorph asText].! !

!PluggableTextMorphPlus methodsFor: 'accessing' stamp: 'ar 7/16/2005 19:01'!
acceptAction
	^acceptAction! !

!PluggableTextMorphPlus methodsFor: 'accessing' stamp: 'ar 7/16/2005 19:01'!
acceptAction: anAction
	acceptAction := anAction! !

!PluggableTextMorphPlus methodsFor: 'styling' stamp: 'ar 8/17/2009 23:48'!
acceptTextInModel 
	
	self okToStyle ifFalse:[^super acceptTextInModel].			
	"#correctFrom:to:with: is sent when the method source is
	manipulated during compilation (removing unused temps,
	changing selectors etc). But 	#correctFrom:to:with: operates 
	on the textMorph's text, and we may be saving an unstyled 
	copy of the text. This means that these corrections will be lost
	unless we also apply the corrections to the unstyled copy that we are saving.
		So remember the unstyled copy in unstyledAcceptText, so
	that when #correctFrom:to:with: is received we can also apply
	the correction to it"
	unstyledAcceptText := styler unstyledTextFrom: textMorph asText.
	[^setTextSelector isNil or:
		[setTextSelector numArgs = 2
			ifTrue: [model perform: setTextSelector with: unstyledAcceptText with: self]
			ifFalse: [model perform: setTextSelector with: unstyledAcceptText]]
	] ensure:[unstyledAcceptText := nil]! !

!PluggableTextMorphPlus methodsFor: 'updating' stamp: 'mt 7/24/2016 13:21'!
applyUserInterfaceTheme

	super applyUserInterfaceTheme.

	self styler ifNotNil: [:s | 
		s reset.
		self updateStyle].! !

!PluggableTextMorphPlus methodsFor: 'accessing' stamp: 'mt 3/30/2015 09:35'!
balloonText
	"Overridden to send selector to model and not self. Do not use #perform:orSendTo: because super does more than just the send.."
	
	self balloonTextSelector ifNotNil: [:selector |
		(self model respondsTo: selector) ifTrue: [
			^ self model perform: selector]].
	
	^ super balloonText! !

!PluggableTextMorphPlus methodsFor: 'styling' stamp: 'ar 8/17/2009 23:49'!
correctFrom: start to: stop with: aString
	"see the comment in #acceptTextInModel "
	unstyledAcceptText ifNotNil:[unstyledAcceptText replaceFrom: start to: stop with: aString ].
	^ super correctFrom: start to: stop with: aString! !

!PluggableTextMorphPlus methodsFor: 'drawing' stamp: 'mt 7/25/2016 11:09'!
drawBalloonTextOn: aCanvas 
	"Show balloon text in the text morph if it has no contents."
	textMorph contents ifNotEmpty: [ ^ self ].
	([ self balloonText ]
		on: Error
		do: [ : err | 'error in balloonText' ]) ifNotNil:
		[ : text | aCanvas
			drawString: text
			in: (self innerBounds insetBy: (5 @ 2 corner: 0 @ 0))
			font: (self userInterfaceTheme balloonTextFont ifNil: [TextStyle defaultFont])
			color: (self userInterfaceTheme balloonTextColor ifNil: [Color gray: 0.7]) ]! !

!PluggableTextMorphPlus methodsFor: 'drawing' stamp: 'mt 3/30/2015 09:30'!
drawOn: aCanvas

	super drawOn: aCanvas.
	self drawBalloonTextOn: aCanvas.! !

!PluggableTextMorphPlus methodsFor: 'accessing' stamp: 'ar 2/11/2005 21:21'!
getColorSelector
	^getColorSelector! !

!PluggableTextMorphPlus methodsFor: 'accessing' stamp: 'ar 2/11/2005 21:21'!
getColorSelector: aSymbol
	getColorSelector := aSymbol.
	self update: getColorSelector.! !

!PluggableTextMorphPlus methodsFor: 'menu' stamp: 'cwp 11/20/2011 23:15'!
getMenu: shiftKeyState
	"Answer the menu for this text view. We override the superclass implementation to
	so we can give the selection interval to the model."
	
	| menu aMenu |
	getMenuSelector == nil ifTrue: [^ nil].
	getMenuSelector numArgs < 3 ifTrue: [^ super getMenu: shiftKeyState].
	menu := MenuMorph new defaultTarget: model.
	getMenuSelector numArgs = 3 ifTrue:
		[aMenu := model 
			perform: getMenuSelector 
			with: menu 
			with: shiftKeyState 
			with: self selectionInterval.
		getMenuTitleSelector ifNotNil: 
			[aMenu addTitle: (model perform: getMenuTitleSelector)].
		^ aMenu].
	^ self error: 'The getMenuSelector must be a 1- or 2 or 3-keyword symbol'! !

!PluggableTextMorphPlus methodsFor: 'accessing' stamp: 'mt 8/16/2016 13:12'!
getTextColorSelector
	^getTextColorSelector! !

!PluggableTextMorphPlus methodsFor: 'accessing' stamp: 'mt 8/16/2016 13:12'!
getTextColorSelector: aSymbol
	getTextColorSelector := aSymbol.
	self update: getTextColorSelector.! !

!PluggableTextMorphPlus methodsFor: 'styling' stamp: 'mt 5/5/2015 14:32'!
hasUnacceptedEdits: aBoolean
	"re-implemented to re-style the text iff aBoolean is true"
	 
	super hasUnacceptedEdits: aBoolean.
	aBoolean ifTrue: [ self updateStyle ].! !

!PluggableTextMorphPlus methodsFor: 'testing' stamp: 'mt 5/6/2015 09:16'!
okToStyle

	^ styler
		ifNil:[false]
		ifNotNil: [:s | model aboutToStyle: s]! !

!PluggableTextMorphPlus methodsFor: 'styling' stamp: 'ar 8/21/2009 21:58'!
setText: aText
	
	self okToStyle ifFalse:[^super setText: aText].
	super setText: (styler format: aText asText).
	aText size < 4096
		ifTrue:[styler style: textMorph contents]
		ifFalse:[styler styleInBackgroundProcess:  textMorph contents]! !

!PluggableTextMorphPlus methodsFor: 'accessing' stamp: 'ar 8/17/2009 23:51'!
styler
	"The styler responsible for highlighting text in the receiver"
	^styler! !

!PluggableTextMorphPlus methodsFor: 'accessing' stamp: 'ar 8/17/2009 23:50'!
styler: anObject
	"The styler responsible for highlighting text in the receiver"
	styler := anObject! !

!PluggableTextMorphPlus methodsFor: 'styling' stamp: 'cmm 7/26/2013 10:39'!
stylerStyled: styledCopyOfText
	"Sent after the styler completed styling the underlying text"
	textMorph contents runs: styledCopyOfText runs .
	"textMorph paragraph recomposeFrom: 1 to: textMorph contents size delta: 0."     "caused chars to appear in wrong order esp. in demo mode. remove this line when sure it is fixed"
	textMorph paragraph composeAll.
	textMorph updateFromParagraph.
	selectionInterval 
		ifNotNil:[
			textMorph editor
				selectInvisiblyFrom: selectionInterval first to: selectionInterval last;
				storeSelectionInParagraph;
				setEmphasisHere].
	textMorph editor blinkParen.
	self scrollSelectionIntoView! !

!PluggableTextMorphPlus methodsFor: 'styling' stamp: 'mt 4/1/2015 15:24'!
stylerStyledInBackground: styledCopyOfText 
	"Sent after the styler completed styling of the text"

	"It is possible that the text string	has changed since the styling began. Disregard the styles if styledCopyOfText's string differs with the current textMorph contents string"
	Project current addDeferredUIMessage: [
		textMorph contents string = styledCopyOfText string
			ifTrue: [self stylerStyled: styledCopyOfText]].! !

!PluggableTextMorphPlus methodsFor: 'updating' stamp: 'mt 8/16/2016 13:14'!
update: what
	what ifNil:[^self].
	what == getColorSelector ifTrue: [self color: (model perform: getColorSelector)].
	what == getTextColorSelector ifTrue: [self setTextColor: (model perform: getTextColorSelector)].
	what == #style ifTrue: [self updateStyle].
	
	
	^super update: what! !

!PluggableTextMorphPlus methodsFor: 'styling' stamp: 'mt 5/5/2015 14:32'!
updateStyle

	self okToStyle
		ifTrue: [styler styleInBackgroundProcess: textMorph contents].! !

!PluggableTextMorphPlus methodsFor: 'initialize' stamp: 'mt 7/3/2016 14:27'!
useDefaultStyler
	"This should be changed to a proper registry but as long as there is only shout this will do"
	
	(self userInterfaceTheme stylerClass ifNil: [Smalltalk at: #SHTextStylerST80])
		ifNotNil: [:stylerClass | self styler: (stylerClass new view: self)].! !

!Morph methodsFor: '*ToolBuilder-Morphic-opening' stamp: 'tpr 10/2/2017 17:06'!
buildWith: aToolBuilder
	"A Morph is already built, so simply return myself"
	^self! !

!Morph methodsFor: '*ToolBuilder-Morphic-opening' stamp: 'mt 10/5/2017 08:40'!
openAsTool

	self openInWorld.! !

SimpleHierarchicalListMorph subclass: #PluggableTreeMorph
	instanceVariableNames: 'rootWrappers selectedWrapper getRootsSelector getChildrenSelector hasChildrenSelector getLabelSelector getIconSelector getSelectedPathSelector setSelectedParentSelector getHelpSelector dropItemSelector wantsDropSelector dragItemSelector dragTypeSelector nodeClass lastKeystrokeTime lastKeystrokes dragStartedSelector doubleClickSelector'
	classVariableNames: 'FilterByLabelsOnly MaximumSearchDepth'
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggableTreeMorph commentStamp: 'ar 2/12/2005 04:38' prior: 0!
A pluggable tree morph.!

!PluggableTreeMorph class methodsFor: 'preferences' stamp: 'cmm 4/13/2015 21:33'!
filterByLabelsOnly
	<preference: 'Filterable Trees by labels only'
		category: 'scrolling'
		description: 'When using the Filterable Lists option, set this to only match the labels, not the contents, of hierarchical lists.  Otherwise, search both labels and contents will be matched.'
		type: #Boolean>
	^ FilterByLabelsOnly ifNil: [ false ]! !

!PluggableTreeMorph class methodsFor: 'preferences' stamp: 'cmm 3/10/2015 16:29'!
filterByLabelsOnly: aBoolean
	FilterByLabelsOnly := aBoolean! !

!PluggableTreeMorph class methodsFor: 'preferences' stamp: 'cmm 3/10/2015 16:38'!
maximumSearchDepth
	<preference: 'Maximum tree search depth'
		category: 'scrolling'
		description: 'When using the Filterable Lists option, this specifies the maximum depth that will be searched below the current selection in of the hierarchy.'
		type: #Number>
	^ MaximumSearchDepth ifNil: [ 1 ]! !

!PluggableTreeMorph class methodsFor: 'preferences' stamp: 'mt 3/8/2015 15:59'!
maximumSearchDepth: anInteger
	MaximumSearchDepth := anInteger.! !

!PluggableTreeMorph methodsFor: 'drag and drop' stamp: 'mt 11/4/2015 12:30'!
acceptDroppingMorph: aTransferMorph event: evt 

	dropItemSelector ifNil: [^ self].
	potentialDropMorph ifNil: [^ self].
	
	model
		perform: dropItemSelector
		withEnoughArguments: {
			aTransferMorph passenger. 
			potentialDropMorph withoutListWrapper. 
			aTransferMorph shouldCopy.
			aTransferMorph}.

	self resetPotentialDropMorph.
	evt hand releaseMouseFocus: self.
	Cursor normal show! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'ar 2/12/2005 00:19'!
balloonTextForNode: node
	getHelpSelector ifNil:[^nil].
	^model perform: getHelpSelector with: node item! !

!PluggableTreeMorph methodsFor: 'event handling' stamp: 'mt 3/13/2015 09:56'!
basicKeyPressed: aChar

	self updateLastKeystrokes: aChar.
	
	model okToChange ifFalse: [^ false].
	
	PluggableListMorph filterableLists
		ifTrue: [self triggerFilterTree]
		ifFalse: [self selectNextMorphByFilter].
	
	^ true! !

!PluggableTreeMorph methodsFor: 'geometry' stamp: 'cmm 3/31/2015 17:01'!
bottomVisibleRowIndex
	| itemAtBottom |
	itemAtBottom := self itemFromPoint: self bottomLeft + (3 @ -3).
	^ itemAtBottom
		ifNil: [ scroller submorphs size ]
		ifNotNil:
			[ : item | scroller submorphs indexOf: item ]! !

!PluggableTreeMorph methodsFor: 'geometry' stamp: 'cmm 3/31/2015 16:46'!
charactersOccluded
	^ self visibleItems
		inject: 0
		into:
			[ : sum : each | sum + each charactersOccluded ]! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'topa 9/23/2016 22:26'!
contentsOfNode: node

	| children |
	getChildrenSelector ifNil: [^#()].
	children := model perform: getChildrenSelector with: node item.
	^ self nodesForChildren: children inNode: node
! !

!PluggableTreeMorph methodsFor: 'event handling' stamp: 'pre 4/26/2017 11:25'!
doubleClick: evt

	| aMorph |
	aMorph := self itemFromPoint: evt position.
	self model perform: self doubleClickSelector with: aMorph withoutListWrapper
! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'pre 4/26/2017 10:22'!
doubleClickEnabled

	^ self doubleClickSelector notNil! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'pre 4/25/2017 11:59'!
doubleClickSelector

	^ doubleClickSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'pre 4/25/2017 11:59'!
doubleClickSelector: anObject

	doubleClickSelector := anObject! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'mvdg 2/11/2007 13:53'!
dragItemSelector
	^dragItemSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'mvdg 2/11/2007 13:53'!
dragItemSelector: aSymbol
	dragItemSelector := aSymbol.
	aSymbol ifNotNil:[self dragEnabled: true].! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'mt 11/4/2015 14:20'!
dragStartedSelector
	^ dragStartedSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'mt 11/4/2015 14:20'!
dragStartedSelector: aSymbol
	dragStartedSelector := aSymbol.! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'mt 6/10/2015 10:05'!
dragTypeSelector
	^dragTypeSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'mt 6/10/2015 10:04'!
dragTypeSelector: aSymbol
	dragTypeSelector := aSymbol.! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:26'!
dropItemSelector
	^dropItemSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'mvdg 2/9/2007 21:37'!
dropItemSelector: aSymbol
	dropItemSelector := aSymbol.
	aSymbol ifNotNil:[self dropEnabled: true].! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'ar 2/12/2005 00:20'!
dropNode: srcNode on: dstNode
	dropItemSelector ifNil:[^nil].
	model perform: dropItemSelector with: srcNode item with: dstNode item! !

!PluggableTreeMorph methodsFor: 'filtering' stamp: 'cmm 4/1/2015 14:02'!
filterTree
	self hasFilter ifFalse:
		[ self removeFilter.
		^ self ].
	self indicateFiltered.
	"Clean up the tree."
	(self selectedMorph
		ifNil: [ self roots ]
		ifNotNil:
			[ : m | {m} ]) do:
		[ : item | | filteredItems |
		item applyFilter: lastKeystrokes.
		item visible ifFalse:
			[ "Do not hide the item where the filter is based on."
			item show.
			item isExpanded ifFalse: [ item toggleExpandedState ] ].
		filteredItems := self filteredItems.
		"If configured as a navigation tool, advance the selection."
		(PluggableTreeMorph maximumSearchDepth = 1 and: [ PluggableTreeMorph filterByLabelsOnly not ]) ifTrue:
			[ |toBeExpanded|
			(filteredItems notEmpty and: [ selectedMorph ~= filteredItems last ]) ifTrue:
				[ self setSelectedMorph:
					(toBeExpanded := selectedMorph
						ifNil: [ filteredItems first ]
						ifNotNil: [ filteredItems after: selectedMorph ]).
				toBeExpanded isExpanded ifFalse: [ toBeExpanded toggleExpandedState ] ] ] ].
	self adjustSubmorphPositions! !

!PluggableTreeMorph methodsFor: 'filtering' stamp: 'cmm 3/31/2015 20:44'!
filteredItems
	"Answer the items which visible because they matched the current filters."
	^ self items select: [ : each | each visible ]! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:24'!
getChildrenSelector
	^getChildrenSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:24'!
getChildrenSelector: aSymbol
	getChildrenSelector := aSymbol.! !

!PluggableTreeMorph methodsFor: 'selection' stamp: 'mt 3/7/2015 12:22'!
getCurrentSelectionItem
	"Our models are supposed to return real objects, not wrappers. See PluggableTreeItemNode."
	
	| selectedObject |
	selectedObject := self getSelectedSelector
		ifNil: [^ nil]
		ifNotNil: [:symbol | model perform: symbol].
	^ scroller submorphs
		detect: [:each | each complexContents item == selectedObject]
		ifFound: [:each | each complexContents]
		ifNone: [nil]! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:26'!
getHelpSelector
	^getHelpSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:26'!
getHelpSelector: aSymbol
	getHelpSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:25'!
getIconSelector
	^getIconSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:25'!
getIconSelector: aSymbol
	getIconSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:25'!
getLabelSelector
	^getLabelSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:25'!
getLabelSelector: aSymbol
	getLabelSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:57'!
getMenuSelector
	^getMenuSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:57'!
getMenuSelector: aSymbol
	getMenuSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:24'!
getRootsSelector
	^getRootsSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:24'!
getRootsSelector: aSelector
	getRootsSelector := aSelector.
	self update: getRootsSelector.! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 03:33'!
getSelectedPathSelector
	^getSelectedPathSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 03:33'!
getSelectedPathSelector: aSymbol
	getSelectedPathSelector := aSymbol.! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'mt 3/7/2015 09:00'!
getSelectedSelector
	^getSelectionSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'mt 3/7/2015 09:00'!
getSelectedSelector: aSymbol
	getSelectionSelector := aSymbol.! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:24'!
hasChildrenSelector
	^hasChildrenSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:25'!
hasChildrenSelector: aSymbol
	hasChildrenSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'filtering' stamp: 'mt 3/8/2015 16:15'!
hasFilter
	^ lastKeystrokes isEmptyOrNil not! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'ar 2/12/2005 00:11'!
hasNodeContents: node
	hasChildrenSelector ifNil:[^node contents isEmpty not].
	^model perform: hasChildrenSelector with: node item! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'ar 2/12/2005 00:20'!
iconOfNode: node
	getIconSelector ifNil:[^nil].
	^model perform: getIconSelector with: node item! !

!PluggableTreeMorph methodsFor: 'filtering' stamp: 'mt 3/9/2015 07:57'!
indicateFiltered
	"self color: (Color gray: 0.95)."! !

!PluggableTreeMorph methodsFor: 'filtering' stamp: 'cbc 2/6/2017 11:12'!
indicateUnfiltered
	self color: (self userInterfaceTheme color ifNil: [Color white]).
	scroller submorphsDo: [:m |
		m visible: true; backgroundColor: nil].
	self adjustSubmorphPositions.! !

!PluggableTreeMorph methodsFor: 'initialization' stamp: 'mt 3/8/2015 16:18'!
initialize

	super initialize.
	
	"Filter"
	lastKeystrokeTime := 0.
	lastKeystrokes := ''! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'topa 9/23/2016 22:25'!
instantiateNodeWithContents: item index: aNumber parent: anObject

	^ (self nodeClass with: item model: self)
		index: aNumber;
		parent: anObject! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'ar 2/12/2005 00:02'!
isDraggableNode: node
	^true! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'cmm 3/31/2015 15:30'!
items
	^ scroller submorphs! !

!PluggableTreeMorph methodsFor: 'event handling' stamp: 'mt 3/8/2015 16:01'!
keyStroke: event

	^ (super keyStroke: event)
		ifTrue: [true]
		ifFalse: [self basicKeyPressed: event keyCharacter].! !

!PluggableTreeMorph methodsFor: 'event handling' stamp: 'mt 4/9/2015 13:48'!
keyboardFocusChange: aBoolean 
	aBoolean ifFalse:
		[ PluggableListMorph clearFilterAutomatically ifTrue:
			[ self hasFilter ifTrue: [ self removeFilter ] ] ].
	super keyboardFocusChange: aBoolean.! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:58'!
keystrokeActionSelector
	^keystrokeActionSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:58'!
keystrokeActionSelector: aSymbol
	keystrokeActionSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'event handling' stamp: 'pre 4/26/2017 10:42'!
mouseDown: evt
	| aMorph selectors |
	
	aMorph := self itemFromPoint: evt position.
	evt yellowButtonPressed  "First check for option (menu) click"
		ifTrue: [
			(PluggableListMorph menuRequestUpdatesSelection and: [model okToChange]) ifTrue: [
				aMorph == selectedMorph 
					ifFalse: [self setSelectedMorph: aMorph]].
			^ self yellowButtonActivity: evt shiftPressed].
	(aMorph notNil and:[aMorph inToggleArea: (aMorph point: evt position from: self)])
		ifTrue:[^self toggleExpandedState: aMorph event: evt]. 
	aMorph ifNil:[^super mouseDown: evt].
	
	aMorph highlightForMouseDown.
	self setProperty: #highlightedMorph toValue: aMorph.
	
	selectors := Array 
		with: #click:
		with: (self doubleClickEnabled ifTrue: [#doubleClick:] ifFalse: [nil])
		with: nil
		with: (self dragEnabled ifTrue:[#startDrag:] ifFalse:[nil]).
	evt hand waitForClicksOrDrag: self event: evt selectors: selectors threshold: HandMorph dragThreshold "pixels".! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'mt 3/6/2015 16:24'!
nodeClass
	^ nodeClass ifNil: [PluggableTreeItemNode]! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'mt 3/6/2015 16:25'!
nodeClass: aListWrapperClass
	nodeClass := aListWrapperClass.! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'topa 9/23/2016 22:26'!
nodesForChildren: aCollection inNode: anObject

	^ aCollection isSequenceable
		ifTrue: [aCollection collectWithIndex: 
			[:item :index |
				self instantiateNodeWithContents: item index: index parent: anObject]]
		ifFalse: [ | count |
			count := 0. "artificial index."
			aCollection collect: 
				[:item |
					count := count + 1.
					self instantiateNodeWithContents: item index: count parent: anObject]]! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'ar 2/12/2005 00:20'!
printNode: node
	getLabelSelector ifNil:[^node item printString].
	^model perform: getLabelSelector with: node item! !

!PluggableTreeMorph methodsFor: 'filtering' stamp: 'mt 3/8/2015 16:38'!
removeFilter
	lastKeystrokes := String empty.
	self indicateUnfiltered.! !

!PluggableTreeMorph methodsFor: 'filtering' stamp: 'mt 3/8/2015 16:21'!
selectNextMorphByFilter! !

!PluggableTreeMorph methodsFor: 'selection' stamp: 'ar 2/12/2005 17:29'!
selectPath: path in: listItem
	path isEmpty ifTrue: [^self setSelectedMorph: nil].
	listItem withSiblingsDo: [:each | 
		(each complexContents item = path first) ifTrue: [
			each isExpanded ifFalse: [
				each toggleExpandedState.
				self adjustSubmorphPositions.
			].
			each changed.
			path size = 1 ifTrue: [
				^self setSelectedMorph: each
			].
			each firstChild ifNil: [^self setSelectedMorph: nil].
			^self selectPath: path allButFirst in: each firstChild
		].
	].
	^self setSelectedMorph: nil

! !

!PluggableTreeMorph methodsFor: 'selection' stamp: 'mt 8/2/2016 14:08'!
selectedMorph: aMorph

	super selectedMorph: aMorph.
		
	"If the update came from the model, make my state consistent again."
	selectedWrapper := aMorph complexContents.
	self setSelectedParentMorph: aMorph.! !

!PluggableTreeMorph methodsFor: 'selection' stamp: 'mt 8/2/2016 14:10'!
setSelectedMorph: aMorph

	selectedWrapper := aMorph complexContents.
	
	"Let the model now about the selected object, not wrapper."
	setSelectionSelector ifNotNil: [:symbol |
		model 
			perform: symbol 
			with: (selectedWrapper ifNotNil: [:w | w item])].
		
	self setSelectedParentMorph: aMorph.! !

!PluggableTreeMorph methodsFor: 'selection' stamp: 'mt 8/2/2016 14:03'!
setSelectedParentMorph: aMorph

	"The model may not have access to the parent object in terms of this tree structure."
	setSelectedParentSelector ifNotNil: [:symbol |
		model
			perform: symbol
			with: (selectedWrapper ifNotNil: [:w | w parent ifNotNil: [:pw | pw item]])].! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'mt 3/7/2015 09:30'!
setSelectedParentSelector
	^ setSelectedParentSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'mt 3/7/2015 09:30'!
setSelectedParentSelector: aSymbol
	setSelectedParentSelector := aSymbol.! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'mt 3/7/2015 08:59'!
setSelectedSelector
	^setSelectionSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'mt 3/7/2015 08:59'!
setSelectedSelector: aSymbol
	setSelectionSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'event handling' stamp: 'mt 3/8/2015 16:38'!
specialKeyPressed: asciiValue

	(super specialKeyPressed: asciiValue)
		ifTrue: [^ true].
		
	(#(8 13) includes: asciiValue) ifTrue: [
		"backspace key" 
		self removeFilter.
		^ true].

	^ false! !

!PluggableTreeMorph methodsFor: 'drag and drop' stamp: 'mt 11/4/2015 18:40'!
startDrag: evt 
	
	| itemMorph |
	self dragItemSelector ifNil: [^ self].
	evt hand hasSubmorphs ifTrue: [^ self].
	self model okToChange ifFalse: [^ self].
	
	itemMorph := scroller submorphs
		detect: [:any | any highlightedForMouseDown]
		ifNone: [^ self].

	"Prepare visuals."
	itemMorph highlightForMouseDown: false.
	self setSelectedMorph: itemMorph.

	[ "Initiate drag."
		(self model perform: self dragItemSelector with: itemMorph withoutListWrapper) ifNotNil: [:passenger | | ddm |
			ddm := (self valueOfProperty: #dragTransferClass ifAbsent: [TransferMorph]) withPassenger: passenger from: self.
			ddm dragTransferType: (self dragTypeSelector ifNotNil: [:s | self model perform: s with: itemMorph withoutListWrapper]).
			ddm updateFromUserInputEvent: evt.
			self dragStartedSelector ifNotNil: [:s | self model perform: s with: itemMorph with: ddm].
			evt hand grabMorph: ddm].
	] ensure: [
		Cursor normal show.
		evt hand releaseMouseFocus: self].! !

!PluggableTreeMorph methodsFor: 'geometry' stamp: 'cmm 3/31/2015 15:24'!
topVisibleRowIndex
	^ scroller submorphs indexOf: (self itemFromPoint: self topLeft+(3@3))! !

!PluggableTreeMorph methodsFor: 'filtering' stamp: 'mt 3/13/2015 09:56'!
triggerFilterTree

	self removeAlarm: #filterTree.
	self addAlarm: #filterTree after: 300.! !

!PluggableTreeMorph methodsFor: 'updating' stamp: 'mt 3/7/2015 09:29'!
update: what
	what ifNil:[^self].
	what == getRootsSelector ifTrue:[
		self wrapRoots: (model perform: getRootsSelector).
		^ self].
	
	what == getSelectedPathSelector ifTrue:[
		self
			selectPath: (model perform: getSelectedPathSelector)
			in: (scroller submorphs at: 1 ifAbsent: [^self]).
		^ self].
		
	what == #expandRootsRequested ifTrue: [
		self expandRoots.
		^ self].
	
	super update: what.
! !

!PluggableTreeMorph methodsFor: 'updating' stamp: 'mt 3/7/2015 11:29'!
update: what with: anObject

	super update: what with: anObject.
	
	what == #objectChanged ifTrue: [
		self updateFromChangedObject: anObject].! !

!PluggableTreeMorph methodsFor: 'updating' stamp: 'mt 3/7/2015 11:50'!
updateFromChangedObject: anObject

	scroller submorphs
		detect: [:morph | morph complexContents item == anObject]
		ifFound: [:morph | self updateMorph: morph]
		ifNone: ["Ignore the request. Object may not be visible anyway."].! !

!PluggableTreeMorph methodsFor: 'event handling' stamp: 'mt 3/8/2015 16:35'!
updateLastKeystrokes: aChar

	| milliseconds slowKeyStroke |
	milliseconds := Time millisecondClockValue.
	slowKeyStroke := (Time milliseconds: milliseconds since: lastKeystrokeTime)
		> (PluggableListMorph filterableLists ifTrue: [ 500 ] ifFalse: [ 300 ]).
	lastKeystrokeTime := milliseconds.
	
	slowKeyStroke
		ifTrue: [lastKeystrokes := aChar asLowercase asString]
		ifFalse: [lastKeystrokes := lastKeystrokes , aChar asLowercase asString.].! !

!PluggableTreeMorph methodsFor: 'updating' stamp: 'mt 3/7/2015 11:54'!
updateMorph: morph

	morph complexContents refresh.
	morph refresh.
	
	morph isExpanded
		ifFalse: [self changed]
		ifTrue: [
			morph
				toggleExpandedState;
				toggleExpandedState.
			self adjustSubmorphPositions].
! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'mt 3/7/2015 11:03'!
updateNode: node
! !

!PluggableTreeMorph methodsFor: 'scrolling' stamp: 'mt 3/8/2015 17:57'!
vUnadjustedScrollRange
	
	| range |
	range := 0.
	scroller submorphsDo: [:m | m visible ifTrue: [range := range + m height]].
	^ range! !

!PluggableTreeMorph methodsFor: 'geometry' stamp: 'cmm 4/1/2015 15:01'!
visibleItems
	^ self items ifNotEmpty:
		[ : items | items
			copyFrom: self topVisibleRowIndex
			to: self bottomVisibleRowIndex ]! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:26'!
wantsDropSelector
	^wantsDropSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:27'!
wantsDropSelector: aSymbol
	wantsDropSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'drag and drop' stamp: 'mt 11/4/2015 14:11'!
wantsDroppedMorph: aTransferMorph event: anEvent

	dropItemSelector ifNil: [^ false].
	wantsDropSelector ifNil: [^ true].

	(aTransferMorph isKindOf: TransferMorph) ifFalse: [^ false].

	^ model perform: wantsDropSelector withEnoughArguments: {
		aTransferMorph passenger.
		aTransferMorph dragTransferType.
		aTransferMorph source.
		aTransferMorph}! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'ar 7/15/2005 12:11'!
wantsDroppedNode: srcNode on: dstNode
	dropItemSelector ifNil:[^false].
	wantsDropSelector ifNil:[^true].
	^(model perform: wantsDropSelector with: srcNode with: dstNode) == true! !

!PluggableTreeMorph methodsFor: 'updating' stamp: 'mt 3/7/2015 10:37'!
wrapRoots: someObjects

	rootWrappers := someObjects collect: [:item|
		self nodeClass with: item model: self].
	self list: rootWrappers.! !

ListItemWrapper subclass: #PluggableTreeItemNode
	instanceVariableNames: 'parent index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggableTreeItemNode commentStamp: 'mt 2/1/2016 10:24' prior: 0!
Tree item for PluggableTreeMorph. My model is the tree morph. I keep track of my parent item and the index in my parent's list of children to support selective refreshing.!

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:16'!
acceptDroppingObject: anotherItem
	^model dropNode: anotherItem on: self! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:16'!
asString
	^model printNode: self! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:16'!
balloonText
	^model balloonTextForNode: self! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:02'!
canBeDragged
	^model isDraggableNode: self! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:03'!
contents
	^model contentsOfNode: self! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:16'!
hasContents
	^model hasNodeContents: self! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:04'!
icon
	^model iconOfNode: self! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'mt 1/24/2016 17:27'!
index

	^ index! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'mt 1/24/2016 17:27'!
index: anInteger

	index := anInteger.! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 01:00'!
item
	^item! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'mt 3/7/2015 10:15'!
parent
	^ parent! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'mt 3/7/2015 10:14'!
parent: aNode
	parent := aNode.! !

!PluggableTreeItemNode methodsFor: 'as yet unclassified' stamp: 'mt 2/1/2016 10:25'!
refresh
	
	self setItem: (self parent contents
		at: self index
		ifAbsent: [self error: 'No support for changed child count yet!!'])
			item.! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:16'!
wantsDroppedObject: anotherItem
	^model wantsDroppedNode: anotherItem on: self! !

SystemWindow subclass: #PluggableSystemWindow
	instanceVariableNames: 'getLabelSelector getChildrenSelector children closeWindowSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggableSystemWindow commentStamp: 'ar 2/11/2005 20:14' prior: 0!
A pluggable system window. Fixes the issues with label retrieval and adds support for changing children.!

!PluggableSystemWindow methodsFor: 'accessing' stamp: 'md 8/31/2005 07:59'!
addPaneMorph: aMorph
	self addMorph: aMorph fullFrame: aMorph layoutFrame! !

!PluggableSystemWindow methodsFor: 'accessing' stamp: 'ar 9/17/2005 21:05'!
closeWindowSelector
	^closeWindowSelector! !

!PluggableSystemWindow methodsFor: 'accessing' stamp: 'ar 9/17/2005 21:05'!
closeWindowSelector: aSymbol
	closeWindowSelector := aSymbol! !

!PluggableSystemWindow methodsFor: 'initialization' stamp: 'cmm 11/19/2015 16:23'!
delete
	(model notNil and: [ closeWindowSelector notNil ]) ifTrue: [ model perform: closeWindowSelector ].
	super delete! !

!PluggableSystemWindow methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:57'!
getChildrenSelector
	^getChildrenSelector! !

!PluggableSystemWindow methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:57'!
getChildrenSelector: aSymbol
	getChildrenSelector := aSymbol! !

!PluggableSystemWindow methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:57'!
getLabelSelector
	^getLabelSelector! !

!PluggableSystemWindow methodsFor: 'accessing' stamp: 'ar 2/13/2005 13:53'!
getLabelSelector: aSymbol
	getLabelSelector := aSymbol.
	self update: aSymbol.! !

!PluggableSystemWindow methodsFor: 'accessing' stamp: 'ar 2/13/2005 13:52'!
label
	^label contents! !

!PluggableSystemWindow methodsFor: 'accessing' stamp: 'ar 2/13/2005 13:51'!
label: aString
	self setLabel: aString.! !

!PluggableSystemWindow methodsFor: 'updating' stamp: 'ar 2/11/2005 20:15'!
update: what
	what ifNil:[^self].
	what == getLabelSelector ifTrue:[self setLabel: (model perform: getLabelSelector)].
	what == getChildrenSelector ifTrue:[
		children ifNil:[children := #()].
		self removeAllMorphsIn: children.
		children := model perform: getChildrenSelector.
		self addAllMorphs: children.
		children do:[:m| m hResizing: #spaceFill; vResizing: #spaceFill].
	].
	^super update: what! !

!SystemWindow methodsFor: '*ToolBuilder-Morphic-opening' stamp: 'mt 10/5/2017 08:45'!
openAsTool
	"Open this window as a tool, that is, honor the preferences such as #reuseWindows and #openToolsAttachedToMouseCursor."
	
	| meOrSimilarWindow |
	meOrSimilarWindow := self openInWorldExtent: self extent.
	(Project uiManager openToolsAttachedToMouseCursor
		and: [ | event |
			event := self currentEvent.
			event isMouse and: [event isMouseUp]])
				ifTrue: [
					meOrSimilarWindow setProperty: #initialDrop toValue: true.
					meOrSimilarWindow hasDropShadow: false.
					self currentHand attachMorph: meOrSimilarWindow].
	^ meOrSimilarWindow
! !

UIManager subclass: #MorphicUIManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!MorphicUIManager commentStamp: 'dtl 5/2/2010 16:07' prior: 0!
MorphicUIManager is a UIManager that implements user interface requests for a Morphic user interface.!

!MorphicUIManager class methodsFor: 'accessing' stamp: 'ar 2/11/2005 15:41'!
isActiveManager
	"Answer whether I should act as the active ui manager"
	^Smalltalk isMorphic! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ul 12/12/2010 22:00'!
chooseClassOrTrait: label from: environment
	"Let the user choose a Class or Trait. Use ListChooser in Morphic."
	
	| names index |
	names := environment classAndTraitNames.
	index := self
		chooseFrom: names
		lines: #()
		title: label.
	index = 0 ifTrue: [ ^nil ].
	^environment
		at: (names at: index)
		ifAbsent: [ nil ]! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 7/16/2005 19:37'!
chooseDirectory: label from: dir
	"Let the user choose a directory"
	^FileList2 modalFolderSelector: dir! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 7/17/2005 00:28'!
chooseFileMatching: patterns label: aString
	"Let the user choose a file matching the given patterns"
	| result |
	result := FileList2 modalFileSelectorForSuffixes: patterns.
	^result ifNotNil:[result fullName]! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 8/30/2009 17:01'!
chooseFont: titleString for: aModel setSelector: setSelector getSelector: getSelector
	"Open a font-chooser for the given model"
	^FontChooserTool default
		openWithWindowTitle: titleString 
		for: aModel 
		setSelector: setSelector 
		getSelector: getSelector! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'mt 8/5/2016 09:42'!
chooseFrom: aList lines: linesArray title: aString 
	"Choose an item from the given list. Answer the index of the selected item."
	
	aList size <= 7 ifTrue: [
		| dialog |
		dialog := DialogWindow new
			title: 'Please Choose';
			message: aString;
			filterEnabled: true;
			autoCancel: true;
			yourself.
		aList doWithIndex: [:ea :index |
			dialog createButton: ea value: index].
		dialog selectedButtonIndex: 1.
		^ dialog getUserResponseAtHand ifNil: [0]].
	
	^ ListChooser chooseFrom: aList title: aString! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'mt 7/21/2016 09:46'!
chooseFrom: labelList values: valueList lines: linesArray title: aString 
	"Choose an item from the given list. Answer the selected item."
	
	| index |
	index := self chooseFrom: labelList lines: linesArray title: aString.
	^ index = 0
		ifTrue: [ nil ]
		ifFalse: [ valueList at: index ]! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'mt 7/19/2016 14:29'!
chooseFromOrAddTo: aList lines: linesArray title: aString

	^ ListChooser
		chooseItemFrom: aList
		title: aString
		addAllowed: true! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'mt 4/14/2015 17:07'!
chooseMultipleFrom: aList lines: linesArray title: aString
	"Choose one or more items from the given list. Answer the indices of the selected items."
	
	^ ListMultipleChooser
		chooseFrom: aList
		title: aString! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'mt 4/14/2015 17:08'!
chooseMultipleFrom: labelList values: valueList lines: linesArray title: aString
	"Choose one or more items from the given list. Answer the selected items."

	^ (ListMultipleChooser
		chooseFrom: labelList
		title: aString) ifNotNil: [:indexList |
			indexList collect: [:index | valueList at: index]]! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 12/12/2009 00:28'!
confirm: queryString
	"Put up a yes/no menu with caption queryString. Answer true if the 
	response is yes, false if no. This is a modal question--the user must 
	respond yes or no."
	^UserDialogBoxMorph confirm: queryString! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 12/12/2009 00:28'!
confirm: aString orCancel: cancelBlock
	"Put up a yes/no/cancel menu with caption aString. Answer true if  
	the response is yes, false if no. If cancel is chosen, evaluate  
	cancelBlock. This is a modal question--the user must respond yes or no."
	^UserDialogBoxMorph confirm: aString orCancel: cancelBlock! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'dtl 3/29/2015 16:59'!
confirm: aString orCancel: cancelBlock title: titleString
	"Put up a yes/no/cancel menu with caption aString, and titleString to label the dialog.
	Answer true if  the response is yes, false if no. If cancel is chosen, evaluate cancelBlock.
	This is a modal question--the user must respond yes or no."
	^ UserDialogBoxMorph
		confirm: aString
		orCancel: cancelBlock
		title: titleString
		at: nil! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'dtl 3/29/2015 14:32'!
confirm: queryString title: titleString
	"Put up a yes/no menu with caption queryString, and titleString to label the dialog.
	Answer true if the response is yes, false if no. This is a modal question--the user
	must respond yes or no."
	^UserDialogBoxMorph confirm: queryString title: titleString! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'dtl 3/29/2015 15:04'!
confirm: queryString title: titleString trueChoice: trueChoice falseChoice: falseChoice
	"Put up a yes/no menu with caption queryString, and titleString to label the dialog.
	The actual wording for the two choices will be as provided in the trueChoice and
	falseChoice parameters. Answer true if the response is the true-choice, false if it
	is the false-choice. This is a modal question -- the user must respond one way or
	the other."
	^ UserDialogBoxMorph confirm: queryString title: titleString trueChoice: trueChoice falseChoice: falseChoice ! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 1/23/2010 14:38'!
confirm: queryString trueChoice: trueChoice falseChoice: falseChoice 
	"Put up a yes/no menu with caption queryString. The actual wording for the two choices will be as provided in the trueChoice and falseChoice parameters. Answer true if the response is the true-choice, false if it's the false-choice.
	This is a modal question -- the user must respond one way or the other."
	^ UserDialogBoxMorph confirm: queryString trueChoice: trueChoice falseChoice: falseChoice ! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'cmm 5/27/2011 14:05'!
displayProgress: titleString at: aPoint from: minVal to: maxVal during: workBlock 
	"Display titleString as a caption over a progress bar while workBlock is evaluated."
	| result progress |
	progress := SystemProgressMorph
		position: aPoint
		label: titleString
		min: minVal
		max: maxVal.
	[ [ result := workBlock value: progress ]
		on: ProgressNotification
		do:
			[ : ex | ex extraParam isString ifTrue:
				[ SystemProgressMorph uniqueInstance
					labelAt: progress
					put: ex extraParam ].
			ex resume ] ] ensure: [ SystemProgressMorph close: progress ].
	^ result! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 7/22/2010 14:27'!
edit: aText label: labelString accept: anAction
	"Open an editor on the given string/text"
	| window |
	window := Workspace open.
	labelString ifNotNil: [ window setLabel: labelString ].
	"By default, don't style in UIManager edit: requests"
	window model
		shouldStyle: false;
		acceptContents:  aText;
		acceptAction: anAction.
	^window.! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 12/12/2009 00:28'!
inform: aString
	"Display a message for the user to read and then dismiss"
	^UserDialogBoxMorph inform: aString! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ul 6/18/2011 11:07'!
informUserDuring: aBlock
	"Display a message above (or below if insufficient room) the cursor 
	during execution of the given block.
		UIManager default informUserDuring:[:bar|
			#(one two three) do:[:info|
				bar value: info.
				(Delay forSeconds: 1) wait]]"
	SystemProgressMorph
		informUserAt: nil during: aBlock.! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 8/4/2009 22:17'!
multiLineRequest: queryString centerAt: aPoint initialAnswer: defaultAnswer answerHeight: answerHeight
	"Create a multi-line instance of me whose question is queryString with
	the given initial answer. Invoke it centered at the given point, and
	answer the string the user accepts.  Answer nil if the user cancels.  An
	empty string returned means that the ussr cleared the editing area and
	then hit 'accept'.  Because multiple lines are invited, we ask that the user
	use the ENTER key, or (in morphic anyway) hit the 'accept' button, to 
	submit; that way, the return key can be typed to move to the next line."
	^FillInTheBlankMorph 
		request: queryString 
		initialAnswer: defaultAnswer 
		centerAt: aPoint 
		inWorld: self currentWorld
		onCancelReturn: nil
		acceptOnCR: false! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 8/4/2009 22:15'!
request: queryString initialAnswer: defaultAnswer 
	"Create an instance of me whose question is queryString with the given 
	initial answer. Invoke it centered at the given point, and answer the 
	string the user accepts. Answer the empty string if the user cancels."
	^FillInTheBlankMorph request: queryString initialAnswer: defaultAnswer ! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'dtl 2/12/2010 19:52'!
request: queryString initialAnswer: defaultAnswer centerAt: aPoint 
	"Create an instance of me whose question is queryString with the given
	initial answer. Invoke it centered at the given point, and answer the
	string the user accepts. Answer the empty string if the user cancels."
	^FillInTheBlankMorph request: queryString initialAnswer: defaultAnswer centerAt: aPoint! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 8/4/2009 22:15'!
requestPassword: queryString
	"Create an instance of me whose question is queryString. Invoke it centered
	at the cursor, and answer the string the user accepts. Answer the empty 
	string if the user cancels."
	^FillInTheBlankMorph requestPassword: queryString! !
